# encoding: utf-8
"""
@project: djangoModel->Auth
@author: 孙楷炎,高栋天
@Email: sky4834@163.com
@synopsis: 小程序SDK
@created_time: 2022/7/7 9:38
"""
from datetime import datetime, timedelta
import json
from logging import getLogger
from pathlib import Path

from django.contrib.auth.hashers import make_password
from django.core.cache import cache
from django.forms import model_to_dict
import jwt
import redis
import requests

from main.settings import BASE_DIR
from xj_role.services.user_group_service import UserGroupService
from .user_detail_info_service import DetailInfoService, write_to_log
from ..models import BaseInfo, Auth, UserSsoToUser
from ..services.user_relate_service import UserRelateToUserService
from ..utils.custom_tool import get_short_id
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.model_handle import parse_model
from ..utils.nickname_generate import gen_one_word_digit

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))

payment_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
payment_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

sub_appid = payment_main_config_dict.wechat_merchant_app_id or payment_module_config_dict.wechat_merchant_app_id or ""
sub_app_secret = payment_main_config_dict.wechat_merchant_app_secret or payment_module_config_dict.wechat_merchant_app_secret or ""
wechat_merchant_name = payment_main_config_dict.wechat_merchant_name or payment_module_config_dict.wechat_merchant_name or ""

app_id = main_config_dict.app_id or module_config_dict.app_id or ""
app_secret = main_config_dict.secret or module_config_dict.secret or ""

app_app_id = payment_main_config_dict.wechat_app_app_id or payment_module_config_dict.wechat_app_app_id or ""

jwt_secret_key = main_config_dict.jwt_secret_key or module_config_dict.jwt_secret_key or ""
expire_day = main_config_dict.expire_day or module_config_dict.expire_day or ""
expire_second = main_config_dict.expire_second or module_config_dict.expire_second or ""

redis_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="main"))
redis_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="main"))

redis_host = redis_main_config_dict.redis_host or redis_module_config_dict.redis_host or ""
redis_port = redis_main_config_dict.redis_port or redis_module_config_dict.redis_port or ""
redis_password = redis_main_config_dict.redis_password or redis_module_config_dict.redis_password or ""

# print(">", sub_appid)
logger = getLogger('log')


class WechatService:
    wx_login_url = "https://api.weixin.qq.com/sns/jscode2session"
    wx_token_url = 'https://api.weixin.qq.com/cgi-bin/token'
    wx_get_phone_url = "https://api.weixin.qq.com/wxa/business/getuserphonenumber"

    def __init__(self):
        self.login_param = {'appid': app_id, 'secret': app_secret, 'grant_type': 'authorization_code'}
        self.redis = redis.Redis(
            host=redis_host,
            port=redis_port,
            password=redis_password
        )

    def get_openid(self, code):
        """
        :param code（openid登录的code）:
        :return:(err,data)
        """
        req_params = {
            'appid': sub_appid,
            'secret': sub_app_secret,
            'js_code': code,
            'grant_type': 'authorization_code',
        }
        user_info = requests.get('https://api.weixin.qq.com/sns/jscode2session', params=req_params, timeout=3,
                                 verify=False)
        return user_info.json()

    def wechat_login(self, phone_code, login_code, sso_serve_id=None, platform_id=None, other_params=None):
        """
        过期续约就是重新登录
        :param other_params:
        :param code: 换取手机号码的code
        :return:(err,data)
        """

        # code 换取手机号
        if other_params is None:
            other_params = {}
        url = self.wx_get_phone_url + "?access_token={}".format(self.get_access_token()['access_token'])
        header = {'content-type': 'application/json'}
        response = requests.post(url, json={'code': phone_code}, headers=header).json()
        if not response['errcode'] == 0:
            return response['errmsg'], ""
        phone = response['phone_info']['phoneNumber']

        if not login_code:
            return None, "微信登录 login_code 必传"
        wechat_openid = self.get_openid(code=login_code)
        if wechat_openid.get("openid", None) is None:
            return None, "获取 openid 失败,请检查code是否过期"
        if not sso_serve_id:
            return None, "平台不能为空"

        openid = wechat_openid.get("openid", "")
        unionid = wechat_openid.get("unionid", "")

        # 通过换取的手机号判断用户是否存在
        current_user = BaseInfo.objects.filter(phone=phone).first()

        # 登录即注册操作
        if not current_user:
            # 用户不存在的时候，进行注册用户
            base_info = {
                'user_name': get_short_id(8),  # 第一次注册的时候给一个唯一的字符串作登录账号
                'nickname': gen_one_word_digit(),
                'phone': phone,
                'email': ''
            }
            BaseInfo.objects.create(**base_info)

            # 注册完成后 重新获取用户信息
            user_info_set = BaseInfo.objects.filter(phone=phone).first()
            user_info = model_to_dict(user_info_set)

            # 生成登录token
            token = WechatService.set_token(user_info.get('id', None), phone, platform_id)
            # 用户第一次登录即注册，允许添加用户的详细信息
            try:
                other_params.setdefault("user_id", user_info.get('id', None))
                other_params.setdefault("score", "5")  # 用户评分初始化，镖行天下业务逻辑 TODO 后期业务抽离，路程控制
                data, detail_err = DetailInfoService.create_or_update_detail(other_params)
                if detail_err:
                    raise Exception(detail_err)
            except Exception as e:
                logger.error('---首次登录写入用户详细信息异常：' + str(e) + '---')

            # 用户第一次登录即注册，绑定用户的分组ID
            try:
                group_id = other_params.get("group_id")
                if group_id:
                    data, err = UserGroupService.user_bind_group(user_id=user_info.get('id', None), group_id=group_id)
                    write_to_log(
                        prefix="group_id:" + str(other_params.get("group_id", "")) + "用户手机号登录，绑定部门ID异常",
                        content=err
                    )
            except Exception as err:
                write_to_log(
                    prefix="用户手机号登录，绑定部门ID异常",
                    content="group_id:" + str(other_params.get("group_id", "")),
                    err_obj=err
                )

            # 检验单点登录信息
            sso_set, sso_err = WechatService.sso_verify(sso_serve_id, user_info.get('id', None), sub_appid, openid,
                                                        unionid)
            if sso_err:
                return None, sso_err

            # 创建用户登录信息，绑定token
            auth_set, auth_err = WechatService.get_token(user_info.get('id', None), token)
            if auth_err:
                return None, auth_err
        else:
            # 用户存在的时候
            user_info = model_to_dict(current_user)

            sso_set, sso_err = WechatService.sso_verify(sso_serve_id, user_info.get('id', None), sub_appid, openid,
                                                        unionid)
            if sso_err:
                return None, sso_err

            token = WechatService.set_token(user_info.get('id', None), phone, platform_id)
            # 修改用户登录信息，绑定token
            auth_set, auth_err = WechatService.get_token(user_id=user_info.get('id', None), token=token,
                                                         is_create=False)
            if auth_err:
                return None, auth_err

        # 绑定用户关系 邀请关系和收益关系
        data, relate_err = UserRelateToUserService.bind_bxtx_relate(params=other_params, user_info=user_info)
        if relate_err:
            logger.error(
                '绑定用户关系异常：' + str(relate_err) +
                ' \n当前用户ID:' + str(user_info.get("id", "")) +
                '\n other_params:' + json.dumps(other_params or {})
            )

        return {'token': auth_set.token, 'user_info': user_info}, None

    @staticmethod
    def wechat_app_login(params):
        unionid = params.get("unionid")
        openid = params.get("openid", None)
        sso_serve_id = params.get("sso_serve_id", 1)
        phone_code = params.get('phone_code', None)
        phone = params.get("phone", None)
        detail_params = params.get("detail_params", None)
        appid = app_app_id
        platform_id = params.get("platform_id", None)
        sso = UserSsoToUser.objects.filter(union_code=unionid, sso_serve_id=sso_serve_id).first()

        if not sso:
            if phone:
                if phone_code is None:
                    return None, "验证码不能为空"
                cache_code = cache.get(phone)
                if phone_code != cache_code:
                    return None, "验证码错误"
                current_user = BaseInfo.objects.filter(phone=phone).first()
                if current_user:
                    user = model_to_dict(current_user)
                    sso_set, sso_err = WechatService.sso_verify(sso_serve_id, user.get('id', None), sub_appid,
                                                                openid, unionid)
                    if sso_err:
                        return None, sso_err

                    token = WechatService.set_token(user.get('id', None), phone, platform_id)
                    auth_set, err = WechatService.get_token(user.get('id', None), token)
                    if err:
                        return None, err

                    return {'token': auth_set.token, 'user_info': user}, None
                else:
                    base_info = {
                        'user_name': get_short_id(8),  # 第一次注册的时候给一个唯一的字符串作登录账号
                        'nickname': gen_one_word_digit(),
                        'phone': phone,
                        'email': '',
                    }
                    base_info = BaseInfo.objects.create(**base_info)
                    if sso_serve_id:
                        sso_set, err = WechatService.sso_verify(sso_serve_id, base_info.id, appid, openid, unionid)
                        if err:
                            return None, err
                    token = WechatService.set_token(base_info.id, phone, platform_id)
                    auth_set, err = WechatService.get_token(current_user.get('id', None), token)
                    if err:
                        return None, err
                    return {'token': auth_set.token, 'user_info': current_user}, None
            else:
                return None, "PHONE_NOT_NULL"
        else:
            sso_dict = model_to_dict(sso)
            current_user = BaseInfo.objects.filter(id=sso_dict.get("user", 0)).first()
            current_user = model_to_dict(current_user)
            token = WechatService.set_token(current_user.get('id', None), current_user.get('phone', None), platform_id)
            # 修改用户登录信息，绑定token
            auth_set, err = WechatService.get_token(user_id=current_user.get('id', None), token=token,
                                                    is_create=False)
            if err:
                return None, err

            # 绑定用户关系 邀请关系和收益关系
            # data, relate_err = UserRelateToUserService.bind_bxtx_relate(params=detail_params, user_info=current_user)
            # if relate_err:
            #     logger.error(
            #         '绑定用户关系异常：' + str(relate_err) +
            #         ' \n当前用户ID:' + str(current_user.get("id", "")) +
            #         '\n detail_params:' + json.dumps(detail_params or {})
            #     )

            return {'token': auth_set.token, 'user_info': current_user}, None

    # 生成单点登录记录
    @staticmethod
    def sso_verify(sso_serve_id, user_id, appid, sso_unicode, union_code):
        """
        生成单点登录记录
        :param sso_serve_id: 单点登录服务ID
        :param user_id: 用户ID
        :param appid: appid
        :param sso_unicode: 单点登录唯一识别码(微信openid)
        :param union_code: union_id
        :return: param_dict
        """
        sso = UserSsoToUser.objects.filter(
            user_id=user_id,
            sso_serve__sso_appid=appid
        ).first()
        if not sso:
            sso_data = {
                "sso_serve_id": sso_serve_id,
                "user_id": user_id,
                "sso_unicode": sso_unicode,
                "union_code": union_code
            }
            sso_set = UserSsoToUser.objects.create(**sso_data)
            if not sso_set:
                return None, "单点登录写入失败"
        sso_set = UserSsoToUser.objects.filter(
            user_id=user_id,
            sso_serve__sso_appid=appid
        ).first()
        if not sso_set:
            return None, "平台用户信息不存在"
        sso_set = model_to_dict(sso_set)
        if not sso_set.get("union_code", None):
            UserSsoToUser.objects.filter(
                user_id=user_id,
                sso_serve__sso_appid=appid
            ).update(union_code=union_code)
        return sso_set, None

    @staticmethod
    def set_token(user_id, account, platform_id):
        # 生成过期时间
        expire_timestamp = datetime.utcnow() + timedelta(
            days=7,
            seconds=0
        )
        # 返回token
        return jwt.encode(
            payload={'user_id': user_id, 'account': account, 'platform_id': platform_id, "exp": expire_timestamp},
            key=jwt_secret_key
        )

    def get_access_token(self):
        param = {
            'appid': sub_appid,
            'secret': sub_app_secret,
            'grant_type': 'client_credential'
        }
        response = requests.get(self.wx_token_url, param).json()
        return response

    # 查询用户信息
    @staticmethod
    def get_user_info(user_id):
        current_user = BaseInfo.objects.filter(id=user_id).first()
        if not current_user:
            return None, "用户信息查询失败"
        current_user = model_to_dict(current_user)
        return current_user, None

    # 绑定token
    @staticmethod
    def get_token(user_id, token, is_create=True):
        current_user = BaseInfo.objects.filter(id=user_id).first()
        current_user = model_to_dict(current_user)
        if is_create:
            auth = {
                'user_id': current_user.get("id", ""),
                'password': make_password('123456', None, 'pbkdf2_sha1'),
                'plaintext': '123456',
                'token': token,
            }
            Auth.objects.update_or_create({'user_id': current_user.get("id", "")}, **auth)
            auth_set = Auth.objects.filter(user_id=current_user.get("id", ""),
                                           token__isnull=False).order_by(
                '-update_time').first()
        else:
            auth = {
                'token': token,
            }
            Auth.objects.update_or_create({'user_id': current_user.get("id", "")}, **auth)
            auth_set = Auth.objects.filter(
                user_id=current_user.get('id', None),
                token__isnull=False
            ).order_by('-update_time').first()

        if not auth_set:
            return None, "密钥生成失败"
        return auth_set, None

    def __del__(self):
        self.redis.close()



# _*_coding:utf-8_*_
from rest_framework.views import APIView

from ..services.user_bank_service import UserBankCardsService
from ..utils.custom_tool import request_params_wrapper
from ..utils.model_handle import util_response
from ..utils.user_wrapper import user_authentication_force_wrapper


# 银行卡管理
class UserBankAPIView(APIView):
    # 卡片列表
    @request_params_wrapper
    def get(self, *args, request_params=None, detail_id=None, **kwargs):
        data, err = UserBankCardsService.get_bank_card(params=request_params, detail_id=detail_id)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    # 添加卡片
    @user_authentication_force_wrapper
    @request_params_wrapper
    def post(self, *args, request_params=None, user_info, **kwargs):
        request_params.setdefault("user_id", user_info.get("user_id"))
        data, err = UserBankCardsService.add(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response()

    # # 修改卡片
    @user_authentication_force_wrapper
    @request_params_wrapper
    def put(self, *args, request_params=None, **kwargs):
        pk = request_params.pop("id", None) or request_params.pop("pk", None)
        data, err = UserBankCardsService.edit(pk=pk, update_params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    # 删除卡片
    @user_authentication_force_wrapper
    @request_params_wrapper
    def delete(self, *args, request_params=None, **kwargs):
        pk = request_params.pop("id", None) or kwargs.pop("pk", None)
        data, err = UserBankCardsService.delete(pk=pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->user_relate_apis
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户的关系方法们
@created_time: 2022/12/13 16:42
"""
from rest_framework.views import APIView

from ..services.user_relate_service import UserRelateTypeService, UserRelateToUserService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper
from ..utils.user_wrapper import user_authentication_force_wrapper


class UserRelateTypeApis(APIView):
    # 关系列表
    @request_params_wrapper
    def get(self, *args, request_params=None, **kwargs):
        data, err = UserRelateTypeService.list(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    # 添加关系
    @request_params_wrapper
    def post(self, *args, request_params=None, **kwargs):
        data, err = UserRelateTypeService.add(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    # 修改关系
    @request_params_wrapper
    def put(self, *args, request_params=None, **kwargs):
        pk = request_params.pop("id", None) or request_params.pop("pk", None)
        data, err = UserRelateTypeService.edit(pk=pk, update_params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    # 删除关系
    @request_params_wrapper
    def delete(self, *args, request_params=None, **kwargs):
        pk = request_params.pop("id", None) or request_params.pop("pk", None)
        data, err = UserRelateTypeService.delete(pk=pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)


class UserRelateToUserApis(APIView):
    # 关系列表
    @request_params_wrapper
    def get(self, *args, request_params=None, **kwargs):
        data, err = UserRelateToUserService.list(
            params=request_params,
            filter_fields=request_params.get("filter_fields", None)
        )
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    # 添加关系
    @request_params_wrapper
    def post(self, *args, request_params=None, **kwargs):
        data, err = UserRelateToUserService.add(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response()

    # # 修改关系
    # @request_params_wrapper
    # def put(self, *args, request_params=None, **kwargs):
    #     pk = request_params.pop("id", None) or request_params.pop("pk", None)
    #     data, err = UserRelateToUserService.edit(pk=pk, params=request_params)
    #     if err:
    #         return util_response(err=1000, msg=err)
    #     return util_response(data=data)

    # 删除关系
    @request_params_wrapper
    def delete(self, *args, request_params=None, **kwargs):
        pk = request_params.pop("id", None) or request_params.pop("pk", None)
        data, err = UserRelateToUserService.delete(pk=pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    # 绑定用户关系
    @user_authentication_force_wrapper
    @request_params_wrapper
    def bind_bxtx_relate(self, *args, request_params=None, user_info=None, **kwargs):
        data, err = UserRelateToUserService.bind_bxtx_relate(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



from django.forms import model_to_dict

from xj_user.utils.custom_tool import write_to_log
from ..models import Platform, UserSsoServe
from ..services.login_service import LoginService


class UserMainService:

    @staticmethod
    def register(params):
        platform_id = params.get("platform_id", None)  # 平台。不应该支持ID传入，无法数据移植。20230507 by Sieyoo
        platform_code = params.get("platform_code", None)
        account = params.get('account', '')
        nickname = params.get('nickname', '')
        phone = params.get('phone', '')
        password = str(params.get('password', ''))

        if platform_code:
            platform_set = Platform.objects.filter(platform_code=platform_code).first()
            if not platform_set:
                return None, "platform不存在平台名称：" + platform_code
            platform_id = model_to_dict(platform_set)['platform_id']

        if platform_id:
            platform_set = Platform.objects.filter(platform_id=platform_id).first()
            if not platform_set:
                return None, "所属平台不存在"
            platform_code = model_to_dict(platform_set)['platform_code']

        account_serv, error_text = LoginService.check_account(account, platform_code, "REGISTER")
        if error_text:
            return None, error_text

        register_serv, register_error = LoginService.register_write(account, nickname, phone, platform_id,
                                                                    platform_code, password)
        if register_error:
            return None, register_error

        return register_serv, None

    """
     登录整合接口，支持以下几种登录方式：
          1、PASSWORD 账户密码登录
          2、SMS 短信验证码登录 （支持多账号登录*）
          3、WECHAT_APPLET 微信小程序授权登录
          4、WECHAT_WEB 微信公众号授权登录
          5、WECHAT_APP 微信APP授权登录

    """

    @staticmethod
    def login_integration_interface(params):
        data = {}
        # ----------------------------获取信息----------------------------------------
        # TODO platform_id字段 即将弃用，改为platform_code 20230507 by Sieyoo
        platform_id = params.get("platform_id", None)  # 平台。不应该支持ID传入，无法数据移植。20230507 by Sieyoo
        platform_code = params.get("platform_code", None)
        user_id = params.get("user_id", None)  # 用户id
        login_type = params.get("login_type", None)  # 支持的登录方式
        code = params.get("code", None)  # 微信登录code
        phone_code = params.get("phone_code", None)  # 微信手机号code
        sms_code = params.get("sms_code", None)  # 短信验证码
        sso_serve_id = params.get("sso_serve_id", 1)  # 单点登录用户平台
        sso_serve_code = params.get("sso_serve_code", None)  # 单点登录用户平台
        phone = params.get("phone", None)  # 手机号
        other_params = params.get("other_params", None)
        account = params.get("account", None)  # 账户
        password = params.get("password", None)  # 密码
        bind_data = params.get("bind_data", None)  # 绑定的数据
        # ------------------------边界检查----------------------------------------------
        if not login_type:
            return None, "登录方式不能为空"
        if platform_code:
            platform_set = Platform.objects.filter(platform_code=platform_code).first()
            if not platform_set:
                return None, "platform不存在平台名称：" + platform_code
            platform_id = model_to_dict(platform_set)['platform_id']

        if platform_id:
            platform_set = Platform.objects.filter(platform_id=platform_id).first()
            if not platform_set:
                return None, "所属平台不存在"
            platform_code = model_to_dict(platform_set)['platform_code']

        if sso_serve_id:
            sso_server_set = UserSsoServe.objects.filter(id=sso_serve_id).first()
            if not sso_server_set:
                return None, "单点登录平台不存在：" + platform_code
            sso_serve_id = model_to_dict(sso_server_set)['id']

        if sso_serve_code:
            sso_server_set = UserSsoServe.objects.filter(sso_code=sso_serve_code).first()
            if not sso_server_set:
                return None, "单点登录平台不存在：" + platform_code
            sso_serve_id = model_to_dict(sso_server_set)['id']
        # ------------------------登录类型判断----------------------------------------------

        if other_params is None:
            other_params = {}
        try:
            current_user, user_err = LoginService.type_judgment(login_type, account, phone, password, platform_code,
                                                                sms_code, user_id, code, phone_code,
                                                                sso_serve_id, bind_data)
            if user_err:
                return None, user_err
            if isinstance(current_user.get("user_info", None), list):
                return current_user, None
            else:
                print(">>>>>", current_user)
                data, err = LoginService.logical_processing(current_user.get("user_info", None),
                                                            current_user.get("phone", None), sso_serve_id,
                                                            current_user.get("appid", None),
                                                            current_user.get("openid", None),
                                                            current_user.get("unionid", None),
                                                            platform_id,
                                                            platform_code, other_params,
                                                            current_user.get("wx_login_type", None))
                if err:
                    return None, err
        except Exception as e:
            write_to_log(
                prefix="登录异常",
                content='---用户登录异常：' + str(e) + '---',
                err_obj=e
            )

        return data, None



# encoding: utf-8
"""
@project: djangoModel->Auth
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 小程序SDK
@created_time: 2022/7/7 9:38
"""
from datetime import datetime, timedelta
import json
from logging import getLogger
from pathlib import Path

from django.contrib.auth.hashers import make_password
import jwt
import requests

from main.settings import BASE_DIR
from xj_user.services.wechat_service import WechatService
from ..models import BaseInfo, Auth, UserSsoToUser
from ..services.user_detail_info_service import DetailInfoService
from ..services.user_relate_service import UserRelateToUserService
from ..utils.custom_tool import get_short_id
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.model_handle import parse_model
from ..utils.nickname_generate import gen_one_word_digit

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))

payment_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
payment_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

sub_appid = payment_main_config_dict.wechat_merchant_app_id or payment_module_config_dict.wechat_merchant_app_id or ""

sub_app_secret = payment_main_config_dict.wechat_merchant_app_secret or payment_module_config_dict.wechat_merchant_app_secret or ""

app_id = main_config_dict.app_id or module_config_dict.app_id or ""
app_secret = main_config_dict.secret or module_config_dict.secret or ""
jwt_secret_key = main_config_dict.jwt_secret_key or module_config_dict.jwt_secret_key or ""
expire_day = main_config_dict.expire_day or module_config_dict.expire_day or ""
expire_second = main_config_dict.expire_second or module_config_dict.expire_second or ""

logger = getLogger('log')


class UserSmsService:

    def get_openid(self, code):
        """
        :param code（openid登录的code）:
        :return:(err,data)
        """
        req_params = {
            'appid': sub_appid,
            'secret': sub_app_secret,
            'js_code': code,
            'grant_type': 'authorization_code',
        }
        user_info = requests.get(
            'https://api.weixin.qq.com/sns/jscode2session',
            params=req_params,
            timeout=3,
            verify=False
        )
        return user_info.json()

    def phone_login(self, phone, login_code, sso_serve_id=None, detail_params=None):
        if detail_params is None:
            detail_params = {}
        # 根据手机号获取用户
        current_user = BaseInfo.objects.filter(phone=phone).filter()
        current_user = parse_model(current_user)
        # 根据 小程序code换取openid
        openid = ""
        if login_code:
            wechat_openid = self.get_openid(code=login_code)
            openid = wechat_openid.get("openid", "")
        if not sso_serve_id:
            return None, "平台不能为空"

        # 如果当前用户不存在则注册在登录授权
        if not current_user:
            # 进行注册用户基础信息
            base_info = {
                'user_name': get_short_id(8),
                'nickname': gen_one_word_digit(),
                'phone': phone,
                'email': '',
                # 'full_name': phone,
            }
            BaseInfo.objects.create(**base_info)
            current_user = BaseInfo.objects.filter(phone=phone).filter()
            current_user = parse_model(current_user)
            current_user = current_user[0]

            # 用户第一次登录即注册，允许添加用户的详细信息
            try:
                detail_params.setdefault("user_id", current_user.get('id', None))
                data, detail_err = DetailInfoService.create_or_update_detail(detail_params)
                if detail_err:
                    raise Exception(detail_err)
            except Exception as e:
                logger.error('---首次登录写入用户详细信息异常：' + str(e) + '---')

            # 进行单点登录相关操作
            # if sso_serve_id:
            #     sso_data = {
            #         "sso_serve_id": sso_serve_id,
            #         "user_id": current_user.get('id', None),
            #         "sso_unicode": openid,
            #         "app_id": sub_appid
            #     }
            #     UserSsoToUser.objects.create(**sso_data)
            if sso_serve_id:
                sso_set, err = WechatService.sso_add(sso_serve_id, current_user.get('id', None),
                                                     wechat_openid['openid'], sub_appid,
                                                     wechat_openid.get("wechat_openid", ""))
                if err:
                    return None, err
            # 生成登录token
            token = self.__set_token(current_user.get('id', None), phone)
            # 创建用户登录信息，绑定token
            auth = {
                'user_id': current_user.get('id', None),
                'password': make_password('123456', None, 'pbkdf2_sha1'),
                'plaintext': '123456',
                'token': token,
            }
            Auth.objects.update_or_create({'user_id': current_user.get('id', None)}, **auth)
            auth_set = Auth.objects.filter(
                user_id=current_user.get('id', None),
                password__isnull=False
            ).order_by('-update_time').first()
            return 0, {'token': auth_set.token, 'user_info': current_user}
        else:
            # 存在当前用户，直接进行授权
            current_user = current_user[0]
            sso = UserSsoToUser.objects.filter(user_id=current_user.get('id', None), app_id=sub_appid).first()
            if not sso:
                # if sso_serve_id:
                #     sso_data = {
                #         "sso_serve_id": sso_serve_id,
                #         "user_id": current_user.get('id', None),
                #         "sso_unicode": openid,
                #         "app_id": sub_appid
                #     }
                #     UserSsoToUser.objects.create(**sso_data)
                sso_set, err = WechatService.sso_add(sso_serve_id, current_user.get('id', None),
                                                     wechat_openid['openid'], sub_appid,
                                                     wechat_openid.get("wechat_openid", ""))
                if err:
                    return None, err
            token = self.__set_token(current_user.get('id', None), phone)
            # 创建用户登录信息，绑定token
            auth = {
                'token': token,
            }
            Auth.objects.filter(user_id=current_user.get('id', None)).update(**auth)
            auth_set = Auth.objects.filter(
                user_id=current_user.get('id', None),
                password__isnull=False
            ).order_by('-update_time').first()

        # 绑定用户关系 邀请关系和收益关系
        # data, relate_err = UserRelateToUserService.bind_bxtx_relate(params=detail_params, user_info=current_user)
        # if relate_err:
        #     logger.error(
        #         '绑定用户关系异常：' + str(relate_err) +
        #         ' \n当前用户ID:' + str(current_user.get("id", "")) +
        #         '\n detail_params:' + json.dumps(detail_params)
        #     )

        return 0, {'token': auth_set.token, 'user_info': current_user}

    def __set_token(self, user_id, account):
        # 生成过期时间
        expire_timestamp = datetime.utcnow() + timedelta(
            days=7,
            seconds=0
        )
        # 返回token
        return jwt.encode(
            payload={'user_id': user_id, 'account': account, "exp": expire_timestamp},
            key=jwt_secret_key
        )



# _*_coding:utf-8_*_

from django.urls import re_path

from .apis import user_cards_apis
from .apis import user_contact_book
from .apis import user_detail_info
from .apis import user_edit
from .apis import user_info
from .apis import user_list
from .apis import user_login
from .apis import user_login_short_message
from .apis import user_login_wechat
from .apis import user_password
from .apis import user_platform
from .apis import user_register
from .apis import user_relate_apis
from .apis import user_sso_serve
from .apis import user_statistics
from .apis.user_add import UserAdd
from .apis.user_login_main import UserLoginMain
from .service_register import register

# 应用名称
app_name = 'xj_user'

register()
# 应用路由
urlpatterns = [
    # 平台相关API
    re_path(r'^platform/?$', user_platform.UserPlatform.as_view(), ),
    re_path(r'^platform_list/?$', user_platform.UserPlatform.list, ),
    re_path(r'^get_user_platform/?$', user_platform.UserPlatform.get_user_platform),  # 获取当前用户的平台信息

    # re_path(r'^register/?$', user_register.UserRegister.as_view(), ),
    re_path(r'^login/?$', user_login.UserLogin.as_view(), ),
    re_path(r'^login_wechat/?$', user_login_wechat.WechetLogin.as_view(), ),
    # re_path(r'^login_wechat_app/?$', user_login_wechat_app.WechetAppLogin.as_view(), ),
    # re_path(r'^login_wechat_h5/?$', user_login_wechat_h5.WechetH5Login.as_view(), ),

    re_path(r'^list/?$', user_list.UserListAPIView.as_view(), ),  # 用户列表
    re_path(r'^info/?$', user_info.UserInfo.as_view(), ),
    re_path(r'^edit/?$', user_edit.UserEdit.user_edit, ),
    # re_path(r'^add/?$', user_add.UserAdd.as_view(), ),  # 管理员添加用户
    re_path(r'^delete/?$', user_edit.UserEdit.as_view(), ),  # 管理员添加用户

    re_path(r'^password/?$', user_password.UserPassword.as_view(), ),
    re_path(r'^contact_book/?$', user_contact_book.UserContactBook.as_view(), ),

    # 详细信息查询/新增/修改group_tree
    re_path(r'^list_detail/?$', user_detail_info.UserListDetail.as_view(), ),
    re_path(r'^detail/?$', user_detail_info.UserDetail.as_view(), ),
    # re_path(r'^detail_add/?$', user_detail_info.UserDetailAdd.as_view(), ),  # 用户必须存在才有信息编辑，所以这个接口是多余的
    re_path(r'^detail_edit/?$', user_detail_info.UserDetailEdit.as_view(), ),
    # re_path(r'^group_tree/?$', user_group_tree.UserGroupTree.as_view(), ),
    re_path(r'^detail_extend_fields/?$', user_detail_info.UserDetailExtendFields.as_view(), ),

    re_path(r'^add/?$', UserAdd.add, ),  # 管理员添加用户

    re_path(r'^login_main/?$', UserLoginMain.login_main, ),  # 登录总接口

    re_path(r'^register/?$', UserLoginMain.register, ),

    re_path(r'^bind_phone/?$', UserLoginMain.bind_phone, ),  # 绑定手机号

    # re_path(r'^send/?$', UserLoginMain.send, ),

    re_path(r'^secondary_authorization/?$', UserLoginMain.secondary_authorization, ),  # 绑定手机号

    # 分组
    # re_path(r'^group/?$', user_group.GroupAPIView.as_view(), ),
    # re_path(r'^group_list/?$', user_group.GroupAPIView.list, ),

    # re_path(r'^send_message/?$', user_short_message.UserShortMessage.as_view(), ),
    # re_path(r'^login_short_message/?$', user_login_short_message.ShortMessageLogin.as_view(), ),
    re_path(r'^js_sdk/?$', user_sso_serve.UserSsoServe.get_js_sdk),  # 微信公众号JS-SDK
    re_path(r'^login_short_message/?$', user_login_short_message.ShortMessageLogin.sms_login),
    re_path(r'^login/?$', user_login.UserLogin.as_view(), ),
    re_path(r'^sso_serve/?$', user_sso_serve.UserSsoServe.as_view(), ),
    re_path(r'^statistics/?$', user_statistics.UserStatisticsAPI.as_view(), ),
    # 用户关系相关接口
    re_path(r'^relate_type/?$', user_relate_apis.UserRelateTypeApis.as_view(), ),
    re_path(r'^relate_user/?$', user_relate_apis.UserRelateToUserApis.as_view(), ),
    # 用户银行卡管理
    re_path(r'^bank_card/?(?P<detail_id>\d+)?$', user_cards_apis.UserBankAPIView.as_view(), ),
    # 镖行业务接口，绑定业务员和邀请人
    re_path(r'^bind_bxtx_relate/?$', user_relate_apis.UserRelateToUserApis.bind_bxtx_relate),
]



# encoding: utf-8
"""
@project: djangoModel->test_login
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 自带的单元测试
@created_time: 2023/4/18 22:06
"""

from django.test import TestCase

from ..services.user_service import UserService


class TestLogin(TestCase):
    user_name = "gaodt"
    password = "123456"
    platform = "镖行天下在线工作平台"

    # def setUp(self):
    #     self.user_name = "gaodt"
    #     self.password = "123456"
    #     self.platform = "镖行天下在线工作平台"

    def test_login(self):
        """正确测试"""
        account_serv, error_text = UserService.check_account(self.user_name)
        assert error_text is None
        user_id = account_serv['user_id']
        auth_serv, error_text = UserService.check_login(
            user_id=user_id,
            password=self.password,
            account=self.user_name,
            platform=self.platform
        )

        assert error_text is None

    def test_with_out_user_name(self):
        """错误操作测试"""
        self.user_name = ""
        account_serv, error_text = UserService.check_account(self.user_name)
        assert error_text is None
        user_id = account_serv['user_id']
        auth_serv, error_text = UserService.check_login(
            user_id=user_id,
            password=self.password,
            account=self.user_name,
            platform=self.platform
        )
        assert error_text is None



# _*_coding:utf-8_*_
from rest_framework.response import Response
from rest_framework.views import APIView

from ..utils.model_handle import parse_data, util_response, JsonResponse
from ..services.user_sso_serve_service import UserSsoServeService


class UserSsoServe(APIView):

    def get(self, request, *args, **kwargs):
        # print(self.request)
        data, err_txt = UserSsoServeService.list()
        if not data:
            return util_response(err=4002, msg=err_txt)
        return Response({
            'err': 0,
            'msg': 'OK',
            'data': data
        })

    def post(self, request, *args, **kwargs):
        # print(self.request)
        params = parse_data(self.request)
        data, err_txt = UserSsoServeService.add(params)
        if not data:
            return util_response(err=4002, msg=err_txt)
        return Response({
            'err': 0,
            'msg': 'OK',
            'data': data
        })
    # 微信公众号JS-SDK
    def get_js_sdk(self):
        params = parse_data(self)
        wxpay_params = UserSsoServeService.get_js_sdk(params)
        return JsonResponse({
            'err': 0,
            'msg': 'OK',
            'data': wxpay_params
        })



# encoding: utf-8
"""
@project: djangoModel->Auth
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 小程序SDK
@created_time: 2022/7/7 9:38
"""
import json
from logging import getLogger
from pathlib import Path

from django.forms import model_to_dict
import requests

from main.settings import BASE_DIR
from xj_role.services.user_group_service import UserGroupService
from xj_user.services.login_service import LoginService
from xj_user.services.wechat_service import WechatService
from ..models import BaseInfo
from ..services.user_detail_info_service import DetailInfoService, write_to_log
from ..services.user_relate_service import UserRelateToUserService
from ..utils.custom_tool import get_short_id
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.nickname_generate import gen_one_word_digit

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))

payment_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
payment_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

sub_appid = payment_main_config_dict.wechat_merchant_app_id or payment_module_config_dict.wechat_merchant_app_id or ""

sub_app_secret = payment_main_config_dict.wechat_merchant_app_secret or payment_module_config_dict.wechat_merchant_app_secret or ""

app_id = main_config_dict.app_id or module_config_dict.app_id or ""
app_secret = main_config_dict.secret or module_config_dict.secret or ""
jwt_secret_key = main_config_dict.jwt_secret_key or module_config_dict.jwt_secret_key or ""
expire_day = main_config_dict.expire_day or module_config_dict.expire_day or ""
expire_second = main_config_dict.expire_second or module_config_dict.expire_second or ""

logger = getLogger('log')


class UserSmsService:

    def get_openid(self, code):
        """
        :param code（openid登录的code）:
        :return:(err,data)
        """
        req_params = {
            'appid': sub_appid,
            'secret': sub_app_secret,
            'js_code': code,
            'grant_type': 'authorization_code',
        }
        user_info = requests.get(
            'https://api.weixin.qq.com/sns/jscode2session',
            params=req_params,
            timeout=3,
            verify=False
        )
        return user_info.json()

    def phone_login(self, phone, login_code, sso_serve_id=None, platform_id=None, other_params=None):
        if other_params is None:
            other_params = {}
        # 根据手机号获取用户
        current_user = BaseInfo.objects.filter(phone=phone).first()
        # print(">>>用户信息 current_user ", current_user)

        # 根据小程序code换取openid
        wechat_openid = self.get_openid(code=login_code)

        if wechat_openid.get("openid", None) is None:
            return None, "获取 openid 失败,请检查code是否过期"

        openid = wechat_openid.get("openid", "")
        unionid = wechat_openid.get("unionid", "")

        if not sso_serve_id:
            return None, "单点登录不能为空"

        if not platform_id:
            return None, "平台不能为空"

        # 如果用户未注册 用手机号进行注册
        if not current_user:
            base_info = {
                'user_name': get_short_id(8),
                'nickname': gen_one_word_digit(),
                'phone': phone,
                'email': '',
            }
            BaseInfo.objects.create(**base_info)

            # 注册完成后 重新获取用户信息
            user_info_set = BaseInfo.objects.filter(phone=phone).first()
            user_info = model_to_dict(user_info_set)

            # 用户第一次登录即注册，允许添加用户的详细信息
            try:
                other_params.setdefault("user_id", user_info.get('id', None))
                other_params.setdefault("score", "5")  # 用户评分初始化，镖行天下业务逻辑
                data, detail_err = DetailInfoService.create_or_update_detail(other_params)
                if detail_err:
                    raise Exception(detail_err)
            except Exception as e:
                write_to_log(prefix="用户手机号登录,绑定详细信息异常", err_obj=e)

            # 用户第一次登录即注册，绑定用户的分组ID
            try:
                group_id = other_params.get("group_id")
                if group_id:
                    data, err = UserGroupService.user_bind_group(user_id=user_info.get('id', None), group_id=group_id)
                    write_to_log(
                        prefix="group_id:" + str(other_params.get("group_id", "")) + "用户手机号登录，绑定部门ID异常",
                        content=err
                    )
            except Exception as err:
                write_to_log(
                    prefix="用户手机号登录，绑定部门ID异常",
                    content="group_id:" + str(other_params.get("group_id", "")),
                    err_obj=err
                )

            # 检验单点登录信息
            sso_set, sso_err = LoginService.sso_verify_2(sso_serve_id, user_info.get('id', None), sub_appid, openid,
                                                          unionid)
            if sso_err:
                return None, sso_err

            # 生成登录token
            token = LoginService.make_token(user_info.get('id', None), phone, platform_id)

            # 创建用户登录信息，绑定token
            auth_set, auth_err = LoginService.bind_token(user_info.get('id', None), token)
            if auth_err:
                return None, auth_err
            return {'token': "Bearer " + auth_set.token, 'user_info': user_info}, None
        # 如果用户已经存在
        else:
            # 存在当前用户，直接进行授权
            user_info = model_to_dict(current_user)
            # 检验单点登录信息
            sso_set, sso_err = LoginService.sso_verify_2(sso_serve_id, user_info.get('id', None), sub_appid, openid,
                                                          unionid)
            if sso_err:
                return None, sso_err

            # 创建用户登录信息，绑定token
            token = LoginService.make_token(user_info.get('id', None), user_info.get('phone', None), platform_id)
            # 修改用户登录信息，绑定token
            auth_set, auth_err = LoginService.bind_token(user_id=user_info.get('id', None), token=token,
                                                         is_create=False)
            if auth_err:
                return None, auth_err

            # 绑定用户关系 邀请关系和收益关系
        # data, relate_err = UserRelateToUserService.bind_bxtx_relate(params=other_params, user_info=user_info)
        # if relate_err:
        #     logger.error(
        #         '绑定用户关系异常：' + str(relate_err) +
        #         ' \n当前用户ID:' + str(user_info.get("id", "")) +
        #         '\n detail_params:' + json.dumps(other_params)
        #     )

        return {'token': "Bearer " + auth_set.token, 'user_info': user_info}, None



# encoding: utf-8
"""
@project: djangoModel->user_auth
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户权限API
@created_time: 2022/8/23 9:16
"""
from rest_framework.views import APIView

from ..services.user_permission_service import PermissionService
from ..services.user_service import UserService
from ..utils.custom_response import util_response


class UserPermissions(APIView):
    def get(self, request):
        module = request.query_params.get("module", "").lower()
        feature = request.query_params.get("feature", "USER_GROUP").upper()
        token = request.META.get('HTTP_AUTHORIZATION', None)
        token_serv, error_text = UserService.check_token(token)
        if error_text:
            return util_response(err=6558, msg=error_text)
        account, user_id = token_serv.values()
        if feature == 'USER_GROUP':
            data, err_text = PermissionService.get_user_group_permission(user_id, module, feature)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->Auth
@author: 孙楷炎,高栋天
@Email: sky4834@163.com
@synopsis: 小程序SDK
@created_time: 2022/7/7 9:38
"""
import uuid
from datetime import datetime, timedelta
import json
from logging import getLogger
from pathlib import Path

import jwt
import redis
import requests
from django.forms import model_to_dict
from django.core.cache import cache
from django.contrib.auth.hashers import make_password
from django.http import HttpResponse

from main.settings import BASE_DIR
from xj_captcha.services.sms_service import SmsService
from xj_role.services.user_group_service import UserGroupService
from xj_user.services.login_service import LoginService
from xj_user.services.user_service import UserService
from xj_user.utils.wechat import get_openid
from .user_detail_info_service import DetailInfoService, write_to_log, re
from ..models import BaseInfo, Auth, UserSsoToUser, Platform
from ..services.user_relate_service import UserRelateToUserService
from ..utils.custom_tool import get_short_id
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.model_handle import parse_model
from ..utils.nickname_generate import gen_one_word_digit

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))

payment_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
payment_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

sub_appid = payment_main_config_dict.wechat_merchant_app_id or payment_module_config_dict.wechat_merchant_app_id or ""
sub_app_secret = payment_main_config_dict.wechat_merchant_app_secret or payment_module_config_dict.wechat_merchant_app_secret or ""
wechat_merchant_name = payment_main_config_dict.wechat_merchant_name or payment_module_config_dict.wechat_merchant_name or ""

app_id = payment_main_config_dict.app_id or payment_module_config_dict.app_id or ""
app_secret = payment_main_config_dict.secret or payment_module_config_dict.secret or ""

subscription_app_id = payment_main_config_dict.wechat_subscription_app_id or payment_module_config_dict.wechat_subscription_app_id or ""
subscription_app_secret = payment_main_config_dict.wechat_subscription_app_secret or payment_module_config_dict.wechat_subscription_app_secret or ""

app_app_id = payment_main_config_dict.wechat_app_app_id or payment_module_config_dict.wechat_app_app_id or ""
app_app_secret = payment_main_config_dict.wechat_app_app_secret or payment_module_config_dict.wechat_app_app_secret or ""

jwt_secret_key = main_config_dict.jwt_secret_key or module_config_dict.jwt_secret_key or ""
expire_day = main_config_dict.expire_day or module_config_dict.expire_day or ""
expire_second = main_config_dict.expire_second or module_config_dict.expire_second or ""

redis_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="main"))
redis_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="main"))

redis_host = redis_main_config_dict.redis_host or redis_module_config_dict.redis_host or ""
redis_port = redis_main_config_dict.redis_port or redis_module_config_dict.redis_port or ""
redis_password = redis_main_config_dict.redis_password or redis_module_config_dict.redis_password or ""

# print(">", sub_appid)
logger = getLogger('log')


class WechatService:
    wx_login_url = "https://api.weixin.qq.com/sns/jscode2session"
    wx_token_url = 'https://api.weixin.qq.com/cgi-bin/token'
    wx_get_phone_url = "https://api.weixin.qq.com/wxa/business/getuserphonenumber"

    def __init__(self):
        self.login_param = {'appid': app_id, 'secret': app_secret, 'grant_type': 'authorization_code'}
        self.redis = redis.Redis(
            host=redis_host,
            port=redis_port,
            password=redis_password
        )

    def get_openid(self, code):
        """
        :param code（openid登录的code）:
        :return:(err,data)
        """
        req_params = {
            'appid': sub_appid,
            'secret': sub_app_secret,
            'js_code': code,
            'grant_type': 'authorization_code',
        }
        user_info = requests.get('https://api.weixin.qq.com/sns/jscode2session', params=req_params, timeout=3,
                                 verify=False)
        return user_info.json()
        # try:
        #     response = requests.get(self.wx_login_url, code).json()
        #     if not response['errcode'] == 0:  # openid 换取失败
        #         return response['errcode'], response['errmsg']
        # except Exception as e:
        #     return 6445, '请求错误'

    def wechat_login(self, phone_code, login_code, sso_serve_id=None, platform_id=None, other_params=None):
        """
        过期续约就是重新登录
        :param other_params:
        :param code: 换取手机号码的code
        :return:(err,data)
        """

        # code 换取手机号
        if other_params is None:
            other_params = {}
        url = self.wx_get_phone_url + "?access_token={}".format(self.get_access_token()['access_token'])
        header = {'content-type': 'application/json'}
        response = requests.post(url, json={'code': phone_code}, headers=header).json()
        if not response['errcode'] == 0:
            return response['errmsg'], ""
        phone = response['phone_info']['phoneNumber']

        if not login_code:
            return None, "微信登录 login_code 必传"
        wechat_openid = self.get_openid(code=login_code)
        if wechat_openid.get("openid", None) is None:
            return None, "获取 openid 失败,请检查code是否过期"
        if not sso_serve_id:
            return None, "平台不能为空"

        openid = wechat_openid.get("openid", "")
        unionid = wechat_openid.get("unionid", "")

        # 通过换取的手机号判断用户是否存在
        current_user = BaseInfo.objects.filter(phone=phone).first()

        platform_set = Platform.objects.filter(platform_id=platform_id).first()
        if not platform_set:
            return None, "所属平台不存在"
        platform_code = model_to_dict(platform_set)['platform_code']

        # 登录即注册操作
        if not current_user:
            # 用户不存在的时候，进行注册用户
            base_info = {
                'user_name': get_short_id(8),  # 第一次注册的时候给一个唯一的字符串作登录账号
                'nickname': gen_one_word_digit(),
                'phone': phone,
                'email': ''
            }
            BaseInfo.objects.create(**base_info)

            # 注册完成后 重新获取用户信息
            user_info_set = BaseInfo.objects.filter(phone=phone).first()
            user_info = model_to_dict(user_info_set)

            # 生成登录token
            token = LoginService.make_token(user_info.get('id', None), phone, platform_id, platform_code)
            # 用户第一次登录即注册，允许添加用户的详细信息
            try:
                other_params.setdefault("user_id", user_info.get('id', None))
                other_params.setdefault("score", "5")  # 用户评分初始化，镖行天下业务逻辑 TODO 后期业务抽离，路程控制
                data, detail_err = DetailInfoService.create_or_update_detail(other_params)
                if detail_err:
                    raise Exception(detail_err)
            except Exception as e:
                logger.error('---首次登录写入用户详细信息异常：' + str(e) + '---')

            # 用户第一次登录即注册，绑定用户的分组ID
            try:
                group_id = other_params.get("group_id")
                if group_id:
                    data, err = UserGroupService.user_bind_group(user_id=user_info.get('id', None), group_id=group_id)
                    write_to_log(
                        prefix="group_id:" + str(other_params.get("group_id", "")) + "用户手机号登录，绑定部门ID异常",
                        content=err
                    )
            except Exception as err:
                write_to_log(
                    prefix="用户手机号登录，绑定部门ID异常",
                    content="group_id:" + str(other_params.get("group_id", "")),
                    err_obj=err
                )

            # 检验单点登录信息
            sso_set, sso_err = LoginService.sso_verify_2(sso_serve_id, user_info.get('id', None), sub_appid, openid,
                                                         unionid)
            if sso_err:
                return None, sso_err

            # 创建用户登录信息，绑定token
            auth_set, auth_err = LoginService.bind_token(user_info.get('id', None), token)
            if auth_err:
                return None, auth_err
        else:
            # 用户存在的时候
            user_info = model_to_dict(current_user)

            sso_set, sso_err = LoginService.sso_verify_2(sso_serve_id, user_info.get('id', None), sub_appid, openid,
                                                         unionid)
            if sso_err:
                return None, sso_err

            token = LoginService.make_token(user_info.get('id', None), phone, platform_id, platform_code)
            # 修改用户登录信息，绑定token
            auth_set, auth_err = LoginService.bind_token(user_id=user_info.get('id', None), token=token,
                                                        is_create=False)
            if auth_err:
                return None, auth_err

        # 绑定用户关系 邀请关系和收益关系
        # data, relate_err = UserRelateToUserService.bind_bxtx_relate(params=other_params, user_info=user_info)
        # if relate_err:
        #     logger.error(
        #         '绑定用户关系异常：' + str(relate_err) +
        #         ' \n当前用户ID:' + str(user_info.get("id", "")) +
        #         '\n other_params:' + json.dumps(other_params or {})
        #     )

        return {'token': auth_set.token, 'user_info': user_info}, None

    # def __make_token(self, user_id, account):
    #     # 生成过期时间
    #     expire_timestamp = datetime.utcnow() + timedelta(
    #         days=7,
    #         seconds=0
    #     )
    #     # 返回token
    #     return jwt.encode(
    #         payload={'user_id': user_id, 'account': account, "exp": expire_timestamp},
    #         key=jwt_secret_key
    #     )
    @staticmethod
    def __make_token(user_id, account):
        # 生成过期时间
        expire_timestamp = datetime.utcnow() + timedelta(
            days=7,
            seconds=0
        )
        # 返回token
        return jwt.encode(
            payload={'user_id': user_id, 'account': account, "exp": expire_timestamp},
            key=jwt_secret_key
        )

    def get_access_token(self):
        # access_token = self.redis.get('access_token')
        # if access_token:
        #     ttl = self.redis.ttl('access_token')
        #     return {"access_token": access_token.decode('utf-8'), 'expires_in': ttl, 'local': True}
        param = {
            'appid': sub_appid,
            'secret': sub_app_secret,
            'grant_type': 'client_credential'
        }
        response = requests.get(self.wx_token_url, param).json()
        # if 'access_token' in response.keys():
        #     self.redis.set('access_token', response['access_token'])
        #     self.redis.expire('access_token', response['expires_in'])
        return response

    @staticmethod
    def login_integration_interface(params):
        # ----------------------------获取信息----------------------------------------
        # TODO platform_id字段 即将弃用，改为platform_code 20230507 by Sieyoo
        platform_id = params.get("platform_id", None)  # 平台。不应该支持ID传入，无法数据移植。20230507 by Sieyoo
        user_id = params.get("user_id", None)  # 用户id
        platform_code = params.get("platform_code", None)
        login_type = params.get("login_type", None)  # 支持的登录方式
        code = params.get("code", None)  # 微信登录code
        phone_code = params.get("phone_code", None)  # 微信手机号code
        sms_code = params.get("sms_code", None)  # 短信验证码
        sso_serve_id = params.get("sso_serve_id", 1)  # 单点登录用户平台
        phone = params.get("phone", None)  # 手机号
        other_params = params.get("other_params", None)
        account = params.get("account", None)  # 账户
        password = params.get("password", None)  # 密码
        bind_data = params.get("bind_data", None)  # 绑定的数据
        sso_app_id = ""
        # ------------------------边界检查----------------------------------------------
        # 关闭。正确的设计是，平台码如果为空，则找到为空的平台码来显示。20230507 by Sieyoo
        # if not platform_id and not platform_code:
        #     return None, "所属平台不能为空"
        if not login_type:
            return None, "登录方式不能为空"
        # if not sso_serve_id:
        #     return None, "单点登录不能为空"

        if platform_code:
            platform_set = Platform.objects.filter(platform_code=platform_code).first()
            if not platform_set:
                return None, "platform不存在平台名称：" + platform_code
            platform_id = model_to_dict(platform_set)['platform_id']

        if platform_id:
            platform_set = Platform.objects.filter(platform_id=platform_id).first()
            if not platform_set:
                return None, "所属平台不存在"
            platform_code = model_to_dict(platform_set)['platform_code']

        # ------------------------登录类型判断----------------------------------------------
        # 初始化
        user_info_set = BaseInfo.objects
        user_info_set = user_info_set.extra(
            select={'register_time': 'DATE_FORMAT(register_time, "%%Y-%%m-%%d %%H:%%i:%%s")'})

        openid = ""
        unionid = ""
        wechat = {}
        try:
            if login_type == "PASSWORD":  # 账号登录

                if re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', account):
                    current_user_count = user_info_set.filter(phone=phone).count()
                    if current_user_count > 1:
                        return None, {"error": "0", "msg": "您输入的用户存在多账号信息，请通过验证码登录",
                                      "wechat_data": {"error": "6666"}}
                account_serv, error_text = UserService.check_account(account)
                if error_text:
                    return None, error_text
                user_id = account_serv['user_id']

                auth_serv, auth_error = UserService.check_login(user_id=user_id, password=password, account=account,
                                                                platform_code=platform_code)
                if auth_error:
                    return None, auth_error

                current_user = user_info_set.filter(id=user_id).first()

            elif login_type == "SMS":  # 短信验证码登录 (比较特殊 支持多用户)
                sms, sms_err = SmsService.check_sms({"phone": phone, "sms_code": sms_code})
                if sms_err and not user_id:
                    return None, sms_err
                current_user_count = user_info_set.filter(phone=phone).count()
                if current_user_count > 1 and not user_id:
                    current_user = user_info_set.extra(select={
                        'avatar': 'SELECT avatar FROM user_detail_info WHERE user_base_info.id = user_detail_info.user_id'})
                    current_user = current_user.filter(phone=phone).values("id", "user_name", "avatar")
                    return {'token': "", 'user_info': list(current_user)}, None
                elif user_id:
                    current_user = user_info_set.filter(id=user_id).first()
                else:
                    current_user = user_info_set.filter(phone=phone).first()

            elif login_type == "WECHAT_APPLET":  # 小程序登录
                wechat['appid'] = sub_appid
                wechat['secret'] = sub_app_secret
                wechat['code'] = code
                wechat['phone_code'] = phone_code
                sso_app_id = wechat['appid']
                wechat_user_info, err = get_openid(login_type, wechat)
                if err:
                    return None, err
                openid = wechat_user_info.get("openid", "")
                unionid = wechat_user_info.get("unionid", "")
                phone = wechat_user_info.get("phone", "")
                sso_set, sso_err = LoginService.sso_verify(sso_serve_id, None, sso_app_id, False,
                                                            openid,
                                                            unionid)
                if sso_set:
                    current_user = user_info_set.filter(id=sso_set['user']).first()
                else:
                    current_user = user_info_set.filter(phone=phone).first()

            elif login_type == "WECHAT_WEB":  # 公众号
                wechat['appid'] = subscription_app_id
                wechat['secret'] = subscription_app_secret
                wechat['code'] = code
                sso_app_id = wechat['appid']
                wechat_user_info, err = get_openid(login_type, wechat)
                if err:
                    return None, err
                openid = wechat_user_info.get("openid", "")
                unionid = wechat_user_info.get("unionid", "")
                if unionid:
                    user_info, err = LoginService.backstepping(unionid)
                    if err:
                        cache.set(openid, {"openid": openid, "unionid": unionid}, 300)  # 5分钟有效期
                        return None, {"error": "0", "msg": "请绑定手机号",
                                      "wechat_data": {"openid": openid, "unionid": unionid, "appid": sso_app_id,
                                                      "error": "50051"}}

                sso_set, sso_err = LoginService.sso_verify(sso_serve_id, None, sso_app_id, False,
                                                            openid,
                                                            unionid)
                current_user = user_info_set.filter(id=sso_set['user']).first()

            elif login_type == "WECHAT_APP":  # APP
                wechat['appid'] = app_app_id
                wechat['secret'] = app_app_secret
                wechat['code'] = code
                sso_app_id = wechat['appid']
                wechat_user_info, err = get_openid(login_type, wechat)
                if err:
                    return None, err
                openid = wechat_user_info.get("openid", "")
                unionid = wechat_user_info.get("unionid", "")
                if unionid:
                    user_info, err = LoginService.backstepping(unionid)
                    if err:
                        cache.set(openid, {"openid": openid, "unionid": unionid}, 300)  # 5分钟有效期
                        return None, {"error": "0", "msg": "请绑定手机号",
                                      "wechat_data": {"openid": openid, "unionid": unionid, "appid": sso_app_id,
                                                      "error": "50051"}}

                sso_set, sso_err = LoginService.sso_verify(sso_serve_id, None, sso_app_id, False,
                                                            openid,
                                                            unionid)
                current_user = user_info_set.filter(id=sso_set['user']).first()

            elif login_type == "BIND":
                if not bind_data:
                    return None, "数据不能为空"

                openid = bind_data.get("openid", "")
                unionid = bind_data.get("unionid", "")
                appid = bind_data.get("appid", "")
                phone = bind_data.get("phone", "")
                sso_app_id = appid
                if not phone:
                    return None, "手机号不能为空"
                sso_set, sso_err = LoginService.sso_verify(sso_serve_id, None, sso_app_id, False,
                                                            openid,
                                                            unionid)
                if sso_set:
                    current_user = user_info_set.filter(id=sso_set['user']).first()
                else:
                    current_user = user_info_set.filter(phone=phone).first()

            else:
                return None, "未支持登录方式"

            # -----------------------逻辑处理-----------------------------------------------

            if other_params is None:
                other_params = {}

            if not current_user:  # 如果不存在则为注册

                base_info = {
                    'user_name': get_short_id(8),
                    'nickname': gen_one_word_digit(),
                    'phone': phone,
                    'email': '',
                    "uuid": uuid.uuid1(),
                    "user_type": "PERSON"
                }
                create_user = BaseInfo.objects.create(**base_info)
                if not create_user:
                    return None, "用户注册失败"
                # 注册完成后 重新获取用户信息
                user_info_set = BaseInfo.objects.filter(id=create_user.id).first()
                user_info = model_to_dict(user_info_set)

                # 生成登录token
                token = LoginService.amke(user_info.get('id', ""), user_info.get("user_name", ""), platform_id,
                                                platform_code)

                try:
                    other_params.setdefault("user_id", user_info.get('id', ""))
                    other_params.setdefault("score", "5")  # 用户评分初始化，镖行天下业务逻辑 TODO 后期业务抽离，路程控制
                    data, detail_err = DetailInfoService.create_or_update_detail(other_params)
                    if detail_err:
                        raise Exception(detail_err)
                except Exception as e:
                    logger.error('---首次登录写入用户详细信息异常：' + str(e) + '---')

                # 用户第一次登录即注册，绑定用户的分组ID
                try:
                    group_id = other_params.get("group_id")
                    if group_id:
                        data, err = UserGroupService.user_bind_group(user_id=user_info.get('id', ""), group_id=group_id)
                        write_to_log(
                            prefix="group_id:" + str(other_params.get("group_id", "")) + "绑定部门ID异常",
                            content=err
                        )
                except Exception as err:
                    write_to_log(
                        prefix="绑定部门ID异常",
                        content="group_id:" + str(other_params.get("group_id", "")),
                        err_obj=err
                    )

                # 检验单点登录信息
                if sso_serve_id:
                    sso_set, sso_err = LoginService.sso_verify(sso_serve_id, user_info.get('id', ""), sso_app_id, True,
                                                            openid, unionid)
                if sso_err:
                    return None, sso_err
                # 创建用户登录信息，绑定token
                auth_set, auth_err = LoginService.bind_token(user_info.get('id', ""), token)
                if auth_err:
                    return None, auth_err
            # ----------------------------------------------------------------------
            else:
                # 用户存在的时候
                user_info = model_to_dict(current_user)
                # print(user_info.get("phone"))

                # 检查单点登录信息
                if sso_serve_id:
                    sso_set, sso_err = LoginService.sso_verify(sso_serve_id, user_info.get('id', ""), sso_app_id, True,
                                                            openid, unionid)
                    if sso_err:
                        return None, sso_err

                # TODO token可以考虑让各个子服务独立获取token，而不是公共生成Token，当然，这样设计好不好有待商考 20230507 by Sieyoo
                token = LoginService.make_token(user_info.get('id', ""), user_info.get("user_name", ""), platform_id,
                                                platform_code)
                # 修改用户登录信息，绑定token
                auth_set, auth_err = LoginService.bind_token(user_id=user_info.get('id', ""), token=token,
                                                             is_create=False)
                if auth_err:
                    return None, auth_err

                # 绑定用户关系 邀请关系和收益关系
                data, relate_err = UserRelateToUserService.bind_bxtx_relate(params=other_params, user_info=user_info)
                if relate_err:
                    write_to_log(
                        prefix='绑定用户关系异常：' + str(relate_err),
                        content='当前用户ID:' + str(user_info.get("id", "")) + '\n detail_params:' + json.dumps(
                            other_params),
                        err_obj=relate_err
                    )

        except Exception as e:
            write_to_log(prefix="用户登录登录异常", err_obj=e)
            return None, "用户登录登录异常:" + str(e)

        return {'token': "Bearer " + auth_set.token, 'user_info': user_info}, None




    def __del__(self):
        self.redis.close()



# encoding: utf-8
"""
@project: djangoModel->user_relate_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户关系服务
@created_time: 2022/12/13 16:45
"""
from django.core.paginator import Paginator, EmptyPage
from django.db.models import F

from xj_role.services.role_service import RoleService
from ..models import UserRelateType, UserRelateToUser, BaseInfo
# 用户关系类型服务
from ..utils.custom_tool import format_params_handle, force_transform_type, filter_fields_handler


class UserRelateTypeService():
    @staticmethod
    def list(params=None):
        if params is None:
            params = {}
        size = params.get("size", 10)
        page = params.get("page", 20)
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["id", "relate_key", "relate_name", ]
        )
        relate_obj = UserRelateType.objects.filter(**filter_params).values()
        count = relate_obj.count()
        page_set = Paginator(relate_obj, size).get_page(page)
        return {'count': count, "page": page, "size": size, "list": list(page_set.object_list)}, None

    @staticmethod
    def add(params=None):
        if params is None:
            params = {}
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["relate_key", "relate_name", "description", "is_multipeople"]
        )
        try:
            relate_obj = UserRelateType.objects.create(**filter_params)
            return {"id": relate_obj.id}, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def edit(pk=None, update_params=None):
        if update_params is None:
            update_params = {}
        filter_params = format_params_handle(
            param_dict=update_params,
            filter_filed_list=["relate_key", "relate_name", "description", "is_multipeople"]
        )
        if not pk or not filter_params:
            return None, "没有可修改的数据"
        try:
            relate_obj = UserRelateType.objects.filter(id=pk)
            if not relate_obj:
                return None, "没有可修改的数据"
            relate_obj.update(**filter_params)

            return None, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def delete(pk=None):
        if not pk:
            return None, "参数错误"
        relate_obj = UserRelateType.objects.filter(id=pk)
        if not relate_obj:
            return None, None
        try:
            relate_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None


# 用户关系映射服务
class UserRelateToUserService():
    @staticmethod
    def list(params=None, filter_fields=None, only_first=False, **kwargs):
        """
        查询用户关系映射
        :param params: 参数
        :param filter_fields: 过滤字段
        :param only_first: 仅仅查询第一条
        """
        # ------------------------- section 参数处理 start ------------------------------------
        params, err = force_transform_type(variable=params, var_type="dict", default={})
        size, err = force_transform_type(variable=params.get("size"), var_type="int", default=10)
        page, err = force_transform_type(variable=params.get("page"), var_type="int", default=1)
        need_pagination, err = force_transform_type(variable=params.get("need_pagination"), var_type="bool", default=True)
        sort = params.get("sort")
        sort = sort if sort and sort in ["created_time", "-created_time", "id", "-id"] else "-created_time"
        default_field_list = [
            "user_id", "with_user_id", "user_relate_type_id", "relate_key", "relate_type_name",
            "user_name", "full_name", "nickname", "with_user_name", "with_full_name", "with_nickname",
            "created_time"
        ]
        filter_fields = filter_fields_handler(
            input_field_expression=filter_fields,
            default_field_list=default_field_list,
            all_field_list=default_field_list + ["user_phone", "with_user_phone"]
        )
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "user_id|int", "user_id_list", "with_user_id|int", "user_relate_type_id|int", "relate_key",
                "user_name", "full_name", "nickname", "with_user_name", "with_full_name", "with_nickname",
                "created_time_start", "created_time_end"
            ],
            alias_dict={
                "user_id_list": "user_id__in", "with_user_id_list": "with_user_id__in",
                "created_time_start": "created_time__gte", "created_time_end": "created_time__lte"
            },
            split_list=["user_id_list", "with_user_id_list"]
        )
        # ------------------------- section 参数处理 end   ------------------------------------

        # ------------------------- section 构建ORM start ------------------------------------
        relate_user_obj = UserRelateToUser.objects.extra(
            select={"created_time": 'DATE_FORMAT(created_time, "%%Y-%%m-%%d %%H:%%i:%%s")'}
        ).annotate(
            relate_key=F("user_relate_type__relate_key"),
            relate_type_name=F("user_relate_type__relate_name"),
            user_name=F("user__user_name"),
            user_phone=F("user__phone"),
            full_name=F("user__full_name"),
            nickname=F("user__nickname"),
            with_user_name=F("with_user__user_name"),
            with_user_phone=F("with_user__phone"),
            with_full_name=F("with_user__full_name"),
            with_nickname=F("with_user__nickname")
        ).filter(**filter_params).order_by(sort).values(*filter_fields)
        # ------------------------- section 构建ORM end ------------------------------------

        # ------------------------- section 构建返回体 start ------------------------------------
        # 单条返回
        if only_first:
            return relate_user_obj.first(), None

        # 列表返回
        total = relate_user_obj.count()
        if not need_pagination and total <= 200:
            return list(relate_user_obj), None

        # 分页返回
        else:
            try:
                page_set = Paginator(relate_user_obj, size).page(page)
            except EmptyPage:
                return {'total': total, "page": page, "size": size, "list": []}, None
            return {'total': total, "page": page, "size": size, "list": list(page_set.object_list)}, None
        # ------------------------- section 构建返回体 end   ------------------------------------

    @staticmethod
    def add(params: dict = None, **kwargs):
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params.update(kwargs)

        # ------------------- section 获取关系类型 start ---------------------------
        relate_key = params.get("user_relate_type_value", params.get("relate_key"))
        if relate_key:
            user_relate_type = UserRelateType.objects.filter(relate_key=relate_key).values().first()
        else:
            user_relate_type = UserRelateType.objects.filter(id=params.get("user_relate_type_id")).values().first()
        if not user_relate_type:
            return None, "不是有效的关系类型"
        params.setdefault("user_relate_type_id", user_relate_type.get("id"))
        # ------------------- section 获取关系类型 end   ---------------------------

        # ------------------- section 过滤字段,并校验合法性 start ---------------------------
        # 参数处理
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["user", "user_id", "with_user", "with_user_id", "user_relate_type", "user_relate_type_id"],
            alias_dict={"user": 'user_id', "with_user": "with_user_id", "user_relate_type": "user_relate_type_id"}
        )
        if filter_params.get("user_id", None) is None or filter_params.get("with_user_id", None) is None or filter_params.get("user_relate_type_id", None) is None:
            return None, "参数错误"
        # ------------------- section 过滤字段,并校验合法性 end   ---------------------------

        # ------------------- section 判断是否可以重复绑定 start ---------------------------
        if user_relate_type.get("is_multipeople"):
            # 检查是否已经绑定过
            relate_user_obj = UserRelateToUser.objects.filter(
                user_id=filter_params['user_id'],
                with_user=filter_params['with_user_id'],
                user_relate_type_id=filter_params['user_relate_type_id']
            ).first()
        else:
            # 可多人绑定的用户关系
            relate_user_obj = UserRelateToUser.objects.filter(
                user_id=filter_params['user_id'],
                user_relate_type_id=filter_params['user_relate_type_id']
            ).first()
        if relate_user_obj:
            return None, "无法重复绑定，或者该关系类型不是多人绑定类型。"
        # ------------------- section 判断是否可以重复绑定 end   ---------------------------

        # ------------------- section IO操作 start ---------------------------
        try:
            relate_user_obj = UserRelateToUser.objects.create(**filter_params)
            return {"id": relate_user_obj.id}, None
        except Exception as e:
            return None, str(e)
        # ------------------- section IO操作 end   ---------------------------

    @staticmethod
    def edit(pk=None, params=None):
        if params is None:
            params = {}
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["user", "user_id", "with_user", "with_user_id", "user_relate_type", "user_relate_type_id"],
            alias_dict={"user": 'user_id', "with_user": "with_user_id", "user_relate_type": "user_relate_type_id"}
        )
        if not pk or not params:
            return None, "没有可修改的数据"

        try:
            relate_user_obj = UserRelateToUser.objects.filter(id=pk)
            if not relate_user_obj:
                return None, "没有可修改的数据"
            relate_user_obj.update(**filter_params)
            return None, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def delete(pk=None):
        if not pk:
            return None, "参数错误"
        relate_user_obj = UserRelateToUser.objects.filter(id=pk)
        if not relate_user_obj:
            return None, None
        try:
            relate_user_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def bind_bxtx_relate(params: dict = None, user_info: dict = None, **kwargs):
        """
        镖行天下绑定用户关系服务
        @note  绑定用户关系 邀请关系和收益关系
        :param params: 请求参数
        :param user_info: 用户信息
        :return: None,err_msg
        """
        user_info, is_pass = force_transform_type(variable=user_info, var_type="dict", default={})
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        params.update(user_info)
        params.update(kwargs)
        # 获取绑定用户关系
        user_id, is_pass = force_transform_type(variable=params.get('user_id', params.get('id')), var_type="int", default=0)  # 当前用户ID
        inviter_id, is_pass = force_transform_type(variable=params.get('inviter_id'), var_type="int", default=0)  # 邀请人ID
        try:
            # 判断是否是一个有效的用户ID
            inviter = BaseInfo.objects.filter(id=inviter_id).first()
            if not inviter:
                return None, None

            # 绑定邀请人
            data, err = UserRelateToUserService.add({
                "user_id": user_id,
                "with_user_id": inviter_id,
                "user_relate_type_value": "invite"
            })
            if err:
                return None, None

            # 邀请人不存在受益人，如果邀请人是业务，则绑定的受益人也是该邀请人
            res, err = RoleService.is_this_role(user_id=inviter_id, role_key="BID-SALESMAN")  # 如果是业务人员
            if res:
                data, err = UserRelateToUserService.add({
                    "user_id": user_id,
                    "with_user_id": inviter_id,
                    "user_relate_type_value": "beneficiary"
                })
                return None, err

            # 查询邀请人的受益人是谁，如果存在则绑定。
            saler = UserRelateToUser.objects.annotate(relate_key=F("user_relate_type__relate_key")).filter(
                user_id=inviter_id, relate_key="beneficiary"
            ).values().first()
            if saler:
                data, err = UserRelateToUserService.add({
                    "user_id": user_id,
                    "with_user_id": saler.get("with_user_id"),
                    "user_relate_type_value": "beneficiary"
                })
                return None, None

        except Exception as e:
            return None, str(e)
        return None, None



# encoding: utf-8
"""
@project: djangoModel->user_detail_info
@author: 孙楷炎
@synopsis: 用户详细信息操作
@created_time: 2022/6/27 19:42
"""
from rest_framework.views import APIView

from ..services.user_detail_info_service import DetailInfoService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper, flow_service_wrapper, dynamic_load_class
from ..utils.user_wrapper import user_authentication_force_wrapper


# 列表
class UserListDetail(APIView):
    @request_params_wrapper
    @user_authentication_force_wrapper
    def get(self, request_params, *args, **kwargs):
        # 参数过滤
        filter_fields = request_params.pop("filter_fields", None)
        data, err_txt = DetailInfoService.get_list_detail(params=request_params, filter_fields=filter_fields)
        if not err_txt:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)


# 用户详细信息
class UserDetail(APIView):
    @user_authentication_force_wrapper
    @request_params_wrapper
    def get(self, *args, request_params=None, user_info=None, **kwargs):
        user_id = request_params.get('user_id') or user_info.get("user_id")
        filter_fields = request_params.get("filter_fields", None)
        data, error_text = DetailInfoService.get_detail(user_id=user_id, filter_fields=filter_fields)
        if error_text is None:
            return util_response(data=data)
        return util_response(err=47767, msg=error_text)


class UserDetailEdit(APIView):

    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def post(self, *args, request_params=None, user_info=None, **kwargs):
        """
        修改用户的详细信息
        """
        if user_info is None:
            user_info = {}
        if request_params is None:
            request_params = {}
        request_params.setdefault('user_id', user_info.get('user_id'))
        # ------------------------ section 查询该用户是否存在详细信息 start -------------------
        if request_params:
            data, err_txt = DetailInfoService.create_or_update_detail(params=request_params.copy())
            if err_txt:
                return util_response(err=47767, msg=err_txt)
        # ------------------------ section 查询该用户是否存在详细信息 end   -------------------

        # ------------------------ section 角色修改联动 start --------------------------------
        try:
            # 编辑角色服务
            RoleService, err = dynamic_load_class(
                import_path="xj_role.services.role_service",
                class_name="RoleService"
            )
            if not err and request_params.get("user_role_list"):
                data, err = RoleService.bind_user_roles(
                    user_id=request_params.get("user_id"),
                    role_list=request_params.get("user_role_list"),
                )

            # # 编辑部门服务
            UserGroupService, err = dynamic_load_class(
                import_path="xj_role.services.user_group_service",
                class_name="UserGroupService"
            )
            if not err and request_params.get("user_group_list"):
                data, err = UserGroupService.user_bind_groups(
                    user_id=request_params.get("user_id"),
                    group_list=request_params.get("user_group_list"),
                )
        except Exception as e:
            pass
        # ------------------------ section 角色修改联动 end   --------------------------------
        return util_response()


# 用户必须存在才有信息编辑，所以这个接口是多余的
class UserDetailExtendFields(APIView):
    def get(self, request, *args):
        # 身份验证，传user_id使用传的，没有传使用token获取的
        # 查询该用户是否存在详细信息 TODO 需要判断修改人是否有权限
        data, err_txt = DetailInfoService.get_extend_fields()
        if err_txt is None:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)



# _*_coding:utf-8_*_

from pathlib import Path
import re

from django.contrib.auth.hashers import make_password
from django.db.models import Q
import jwt
from rest_framework import response
from rest_framework.permissions import AllowAny
from rest_framework.views import APIView

from main.settings import BASE_DIR
from ..models import *
from ..services.user_detail_info_service import get_short_id
from ..utils.custom_tool import request_params_wrapper
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict

module_dir = Path(__file__).parent.parent
root_config = JDict(JConfig.get_section(str(BASE_DIR) + '/config.ini', 'xj_user', encode='utf-8-sig'))
module_config = JDict(JConfig.get_section(str(module_dir) + '/config.ini', 'xj_user', encode='utf-8-sig'))


# 用户注册
class UserRegister(APIView):
    permission_classes = (AllowAny,)
    model = BaseInfo
    params = None

    @request_params_wrapper
    def post(self, *args, request_params, **kwargs, ):
        if request_params is None:
            request_params = {}
        self.params = request_params  # 返回QueryDict类型
        token = None
        try:
            account = str(self.params.get('account', ''))
            password = str(self.params.get('password', ''))
            platform_code = str(self.params.get('platform_code', ''))
            phone = str(self.params.get('phone', ''))
            full_name = str(self.params.get('full_name', ''))
            # 边界检查
            if not account:
                raise MyApiError("账户名必填", 2001)
            if not password:
                raise MyApiError("密码必填", 2003)
            # if not full_name:
            #     raise MyApiError("姓名必填", 2008)

            # 检查平台是否存在
            platform_set = Platform.objects.filter(platform_code__iexact=platform_code)
            if platform_set.count() == 0:
                raise MyApiError("platform不存在平台名称：" + platform_code, 2009)
            platform_id = platform_set.first().platform_id

            # 账号类型判断
            if re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', account):
                account_type = 'phone'
            elif re.match(r'^\w+[\w\.\-\_]*@\w+[\.\w]*\.\w{2,}$', account):
                account_type = 'email'
            elif re.match(r'^[A-z\u4E00-\u9FA5]+\w*$', account):
                account_type = 'username'
            else:
                raise MyApiError("账号必须是用户名、手机或者邮箱，用户名不能是数字开头", 2009)
            # 检查账号是否存在
            user_list = None
            # if account_type == 'phone':
            #     user_list = BaseInfo.objects.filter(Q(phone=account))
            # el
            if account_type == 'email':
                user_list = BaseInfo.objects.filter(Q(email=account))
            elif account_type == 'username':
                user_list = BaseInfo.objects.filter(Q(user_name=account))
            # 账号验重
            # if user_list.count() and account_type == 'phone':
            #     raise MyApiError("手机已被注册: " + account)
            # el
            if user_list.count() and account_type == 'email':
                raise MyApiError("邮箱已被注册: " + account)
            elif user_list.count() and account_type == 'username':
                raise MyApiError("用户名已被注册: " + account)
            # 创建用户
            base_info = {
                # 'platform_uid': round(random.random()*1000000000),
                # 'platform_id': platform_id,
                'user_name': account if account_type == 'username' else get_short_id(),
                'phone': phone,
                'email': account if account_type == 'email' else '',
                # 'full_name': full_name,
            }
            current_user = BaseInfo.objects.create(**base_info)

            if platform_id:
                platforms_users = PlatformsToUsers.objects.create(**{
                    "platform_id": platform_id,
                    "user_id": current_user.id
                })
                if not platforms_users:
                    return None, "平台写入失败"
            # SECURITY WARNING: keep the secret key used in production secret!
            SECRET_KEY = 'django-insecure-l1$-m!u=!f9&o2$f(cm#dasus&a=5i1#+kh)090_=p%+==%9o1'
            JWT_SECRET_KEY = '@xzm2021!'
            c1 = root_config
            c2 = module_config
            jwt_secret_key = c1.jwt_secret_key or c2.jwt_secret_key or "@zxmxy2021!"
            token = jwt.encode({'account': account}, jwt_secret_key)
            auth = {
                'user_id': current_user.id,
                'password': make_password(password, None, 'pbkdf2_sha1'),
                'plaintext': password,
                'token': token,
            }
            current_auth = Auth.objects.create(**auth)
            res = {
                'err': 0,
                'msg': '注册成功',
                'data': {
                    "user_id": current_user.id,
                    "token": token,
                },
            }

        except SyntaxError:
            res = {
                'err': 4001,
                'msg': '语法错误'
            }
        except LookupError:
            res = {
                'err': 4002,
                'msg': '无效数据查询'
            }
        # 这里 result是一个类的对象，要用result.属性名来返回
        except Exception as valueError:
            res = {
                'err': valueError.err if hasattr(valueError, 'err') else 4000,
                'msg': valueError.msg if hasattr(valueError, 'msg') else valueError.args,
            }
        except:
            res = {
                'err': 4999,
                'msg': '未知错误'
            }
        headers = {"Authorization": token, }
        return response.Response(data=res, status=None, template_name=None, headers=headers, content_type=None)

    # def put(self, request, *args, **kwargs):
    #     self.params = parse_data(request)
    #     detaili_list = DetailInfoService.transform_result(self.params)
    #     detaili = detaili_list[0]
    #     # 添加逻辑
    #     try:
    #         user_name = str(self.params.get('user_name', ''))
    #         full_name = str(self.params.get('full_name', ''))
    #         phone = str(self.params.get('phone', ''))
    #         nickname = str(self.params.get('nickname', ''))
    #         password = str(self.params.get('password', ''))
    #         # 用户角色部门绑定
    #         user_role_list = self.params.get('user_role_list', None)
    #         user_group_list = self.params.get('user_group_list', None)
    #
    #         base_info = {}
    #         if user_name:
    #             base_info['user_name'] = user_name
    #             is_exists = BaseInfo.objects.filter(user_name=user_name).exists()
    #             if is_exists:
    #                 return util_response('', 7557, "用户名已存在")
    #
    #         if full_name:
    #             base_info['full_name'] = full_name
    #         if nickname:
    #             base_info['nickname'] = nickname
    #         else:
    #             base_info['nickname'] = gen_one_word_digit()
    #         if phone:
    #             base_info['phone'] = phone
    #
    #         current_user = BaseInfo.objects.create(**base_info)
    #
    #         c1 = root_config
    #         c2 = module_config
    #         jwt_secret_key = c1.jwt_secret_key or c2.jwt_secret_key or "@zxmxy2021!"
    #         token = jwt.encode({'account': user_name}, jwt_secret_key)
    #
    #         if password:
    #             auth = {
    #                 'user_id': current_user.id,
    #                 'password': make_password(password, None, 'pbkdf2_sha1'),
    #                 'plaintext': password,
    #                 'token': token,
    #             }
    #             Auth.objects.create(**auth)
    #
    #         if detaili.get("user_name"):
    #             del detaili['user_name']
    #         if detaili.get("full_name"):
    #             detaili['real_name'] = detaili['full_name']
    #             del detaili['full_name']
    #         if detaili.get("nickname"):
    #             del detaili['nickname']
    #         if detaili.get("phone"):
    #             del detaili['phone']
    #         if detaili.get("email"):
    #             del detaili['email']
    #         if detaili.get("user_role_list"):
    #             del detaili['user_role_list']
    #         if detaili.get("user_group_list"):
    #             del detaili['user_group_list']
    #         detaili = DetailInfoService.transform_params(detaili)
    #         detailinfo = DetailInfo.objects.filter(user_id=current_user.id).first()
    #         if detailinfo:
    #             DetailInfo.objects.filter(user_id=current_user.id).update(**detaili)
    #         else:
    #             detaili['user_id'] = current_user.id
    #             DetailInfo.objects.create(**detaili)
    #
    #         # 用户绑定权限和部门
    #         if user_group_list:
    #             UserGroupService.bind_user_group(current_user.id, user_group_list)
    #
    #         if user_role_list:
    #             RoleService.bind_user_role(current_user.id, user_role_list)
    #
    #         res = {
    #             'err': 0,
    #             'msg': '创建成功',
    #             'data': {"user_id": current_user.id},
    #         }
    #
    #     except SyntaxError:
    #         res = {
    #             'err': 4001,
    #             'msg': '语法错误'
    #         }
    #     except LookupError:
    #         res = {
    #             'err': 4002,
    #             'msg': '无效数据查询'
    #         }
    #     except Exception as valueError:
    #         res = {
    #             'err': valueError.err if hasattr(valueError, 'err') else 4000,
    #             'msg': valueError.msg if hasattr(valueError, 'msg') else valueError.args,
    #         }
    #     except:
    #         res = {
    #             'err': 4999,
    #             'msg': '未知错误'
    #         }
    #
    #     return response.Response(data=res, status=None, template_name=None, headers={"Authorization": token},
    #                              content_type=None)


class MyApiError(Exception):
    def __init__(self, message, err_code=4010):
        self.msg = message
        self.err = err_code

    def __str__(self):
        # repr()将对象转化为供解释器读取的形式。可省略
        return repr(self.msg)



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""
from cmath import cos
import copy
import datetime
import difflib
import importlib
import inspect
import json
from logging import getLogger
from math import sin, asin, sqrt
import random
import re
import sys
import time
from urllib.parse import parse_qs
import uuid

from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.wsgi import WSGIRequest
from numpy.core._multiarray_umath import radians
from rest_framework.request import Request
import xmltodict

# =========== section 单例装饰器 start ==================


_instance = {}


# 单例装饰器
def singleton_decorator(cls, *args, **kwargs):
    def inner(*args, **kwargs):
        cls_name = cls.__name__
        if cls_name not in _instance:
            _instance[cls_name] = cls(*args, **kwargs)
        return _instance[cls_name]

    return inner


# 获取单例
def get_singleton(key):
    return _instance.get(key, {})


# =========== section 单例装饰器 end    ==================


def dynamic_load_class(import_path: str = None, class_name: str = None, find_services=False):
    """
    动态加载模块中的类,返回类的指针
    可从通过RPC协议从consul服务获取其他服务器的服务类。
    :param find_services: 是否使用RPC，发现服务
    :param import_path: 导入类的文件路径
    :param class_name: 导入文件中的哪一个类
    :return: class_instance,err_msg
    """
    try:
        class_instance = getattr(sys.modules.get(import_path), class_name, None)
        if class_instance is None:
            import_module = importlib.import_module(import_path)
            class_instance = getattr(import_module, class_name)
        copy_class_instance = copy.deepcopy(class_instance)
        return copy_class_instance, None
    except AttributeError:
        return None, "系统中不存在该模块"
    except Exception as e:
        return None, str(e)


def dynamic_load_function(import_path: str = None, function_name=None, find_services=False):
    """
    动态加载模块中的类,返回类的指针
    可从通过RPC协议从consul服务获取其他服务器的服务类。
    :param find_services: 是否使用RPC，发现服务
    :param import_path: 导入类的文件路径
    :param function_name: 导入文件中方法名称
    :return: class_instance,err_msg
    """
    try:
        function_instance = getattr(sys.modules.get(import_path), function_name, None)
        if function_instance is None:
            import_module = importlib.import_module(import_path)
            function_instance = getattr(import_module, function_name)
        return function_instance, None
    except AttributeError:
        return None, "系统中不存在该方法"
    except Exception as e:
        return None, str(e)


def is_number(s):
    """识别任何语言的数字字符串"""
    try:
        float(s)
        return True
    except ValueError:
        pass

    try:
        import unicodedata
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass

    return False


# json 结果集返回
def parse_json(result):
    if not result is None:
        if type(result) is str:
            try:
                result = json.loads(result.replace("'", '"').replace('\\r', "").replace('\\n', "").replace('\\t', "").replace('\\t', ""))
            except Exception as e:
                return result
        if type(result) is list:
            for index, value in enumerate(result):
                result[index] = parse_json(value)
        if type(result) is dict:
            for k, v in result.items():
                result[k] = parse_json(v)
    return result


# 函数参数解析，返回变量字典
def service_params_adapter(func, params):
    """
    获取函数的参数值，并且逐一进行赋值
    如果有**kwarg则直接进行拆包传值如：**params
    """
    res_dict = {}

    has_var_keyword = False
    inspect_obj = inspect.signature(func)
    for k, v in inspect_obj.parameters.items():
        if v.kind == inspect.Parameter.VAR_KEYWORD:
            has_var_keyword = True
        res_dict[k] = __service_params_adapter_handle(k, v, params)
    res_dict = params if has_var_keyword else res_dict
    return res_dict


# 判断参数是否有默认值
def __service_params_adapter_handle(params_name, param_obj, params):
    """
    判断参数是否有默认值
    如果有默认值并且没有传这个参数则使用默认值，有值返回该值，没有默认值并且没有传值则返回None
    :param params_name:
    :param param_obj:
    :param params:
    :return: res
    """
    res = None
    if not param_obj.default == inspect._empty:
        res = param_obj.default
    if params.get(params_name):
        res = params.get(params_name)
    return res


# 过滤list内容，白名单、黑名单、别名
def format_list_handle(param_list, filter_filed_list=None, remove_filed_list=None, alias_dict=None, remove_repeat=True):
    """
    过滤list内容
    :param param_list: 传入 param_list
    :param filter_filed_list: 需要的字段
    :param remove_filed_list: 需要删除的列表
    :param alias_dict: 元素起别名
    :return:param_list： 处理后的 param_list
    """
    if not param_list:
        return param_list
    # 类型判断 过滤字段
    if filter_filed_list and isinstance(filter_filed_list, list):
        param_list = [i for i in param_list if i in filter_filed_list]

    # 类型判断， 剔除字段
    if remove_filed_list and isinstance(remove_filed_list, list):
        param_list = [j for j in param_list if not j in remove_filed_list]

    # 类型判断 字段转换
    if alias_dict and isinstance(alias_dict, dict):
        param_list = [alias_dict.get(k, k) for k in param_list]

    # 进行去重
    if remove_repeat:
        param_list = list(set(param_list))

    return param_list


# 强制转换类型数据
def force_transform_type(variable=None, var_type: str = None, default=None, **kwargs):
    """
    强制转换类型，转换规则如下：
    1.变量为空使用默认值返回。认为是空值判断。
    2.转换类型为空则，返回输入变量。认为不需要进行类型转换。
    3.无法转换则返回默认值，转换失败，数据存在问题则饭hi默认值。
    :param default: 数据为空或无法转换的时候返回默认值
    :param variable: 变量
    :param var_type: 转换类型 str|int|bool|float|dict|list|date
    :return: transform_variable, is_err
    """
    if variable is None:  # 为空时候，则返回默认值
        return default, None
    if not var_type:  # 转换类型为空，则不进行转换
        return variable, None
    # 强制类型转换
    try:
        if var_type == "str":
            return variable if isinstance(variable, str) else str(variable), None
        elif var_type == "int":
            return variable if isinstance(variable, int) else int(variable), None
        elif var_type == "bool":
            if variable in (None, '', [], (), {}):
                return None, None
            if variable in (True, False):
                return bool(variable), None
            if variable in ('t', 'True', '1'):
                return True, None
            if variable in ('f', 'False', '0'):
                return False, None
            return None, "'" + str(variable) + "'" + "不是一个有效的布尔类型数据"
        elif var_type == "float":
            return variable if isinstance(variable, float) else float(variable), None
        elif var_type == "only_dict":
            # 不包括列表
            variable = parse_json(variable)
            if isinstance(variable, dict):
                return variable, None
            return default, str(variable) + "无法转换为json格式"
        elif var_type == "dict":
            # 字典包括列表字典
            variable = parse_json(variable)
            if isinstance(variable, dict) or isinstance(variable, list):
                return variable, None
            return default, str(variable) + "无法转换为json格式"
        elif var_type == "list":
            variable = parse_json(variable)
            if isinstance(variable, list):
                return variable, None
            return default, str(variable) + "无法转换为list格式"
        elif var_type == "list_int":
            variable = parse_json(variable)
            result = []
            if isinstance(variable, list):
                for i in variable:
                    # 过滤掉非int类型
                    i, is_pass = force_transform_type(variable=i, var_type="int")
                    if is_pass:
                        continue
                    result.append(i)
                return result, None
            return default, str(variable) + "无法转换为list格式"
        elif var_type == "date":
            format_map = {
                '%Y-%m-%d': "%Y-%m-%d 00:00:00",
                '%Y-%m-%d %H': '%Y-%m-%d %H:00:00',
                '%Y-%m-%d %H:%M': '%Y-%m-%d %H:%M:00',
                '%Y-%m-%d %H:%M:%S': '%Y-%m-%d %H:%M:%S'
            }
            for format_input, format_output in format_map.items():
                try:
                    return datetime.datetime.strptime(variable, format_input).strftime(format_output), None
                except ValueError:
                    pass
            raise ValueError
        else:  # 如果指派的类型不存在，默认值非空则返回默认值，否则原样返回
            return variable if default is None else default, str(var_type) + "不是一个有效的转换类型"
    except TypeError:
        return default, "'" + str(variable) + "'" + "不是一个有效的" + str(var_type) + "类型数据"
    except ValueError:
        return default, "'" + str(variable) + "'" + "不是一个有效的" + str(var_type) + "类型数据"


# 处理字典：白名单、黑名单、别名、移除空值、筛选类型
def format_params_handle(
        param_dict: dict,
        filter_filed_list: list = None,
        remove_filed_list: list = None,
        alias_dict: dict = None,
        split_list: list = None,
        split_char: str = ";",
        is_remove_null: bool = True,
        is_remove_empty: bool = False,
        date_format_dict: dict = None,
        is_validate_type: bool = False,
) -> dict:
    """
    字段筛选并替换成别名
    :param is_validate_type: 是否验证数据合法性
    :param param_dict: 参数值
    :param filter_filed_list: 字段白名单，如:["id":int]
    :param remove_filed_list: 字段黑名单,["id",....]
    :param alias_dict: 别名字典, {"id":"user_id"}
    :param split_char: 字符串拆分依据
    :param split_list: 拆分字符换,该参数为了适配使用符号分割成列表。["id_list",....]
    :param is_remove_null:  是否把带有None的值移除掉
    :param is_remove_empty: 是否移除value类型等于False的字段,移除"",0,"0",None
    :param date_format_dict: 日期格式化 日期列表，{field：(from_format,to_format),}如：{"field":("%Y-%m-%d %H:%M:%S","%Y-%m-%d %H:%M:%S")}
    :return: param_dict
    """
    # 转换的数据类型不符合，直接返回出去
    if not isinstance(param_dict, dict):
        raise ValueError("param_dict 必须是字典格式")
    # 过滤字段并且得数据类型映射
    new_filter_filed_list = []  # 拆分字段类型的白名单
    must_type_map = {}  # 从白名单中拆分出字段与类型的映射
    if filter_filed_list and isinstance(filter_filed_list, list):
        for i in filter_filed_list:
            key_type_list = i.split("|", 1)
            [key, key_type] = key_type_list if len(key_type_list) == 2 else [key_type_list[0], None]  # 如果为类型None，则默认是字符串
            must_type_map[key] = key_type
            new_filter_filed_list.append(key)
        param_dict = {k: v for k, v in param_dict.copy().items() if k in new_filter_filed_list and (not is_remove_empty or v) and (not is_remove_null or not v is None)}
    # 剔除字段
    if remove_filed_list and isinstance(remove_filed_list, list):
        param_dict = {k: v for k, v in param_dict.copy().items() if not k in remove_filed_list and (not is_remove_empty or v) and (not is_remove_null or not v is None)}
    # 字段拆分
    if split_list and isinstance(split_list, list):
        param_dict = {k: (str(v).split(split_char) if k in split_list and not isinstance(v, list) else v) for k, v in param_dict.copy().items()}
    # 类型转换
    if must_type_map and isinstance(must_type_map, dict):
        for k, v in param_dict.copy().items():
            v, is_err = force_transform_type(v, must_type_map.get(k, None))
            if is_err and not is_validate_type:
                param_dict.pop(k)
            elif is_err and is_validate_type:
                raise ValueError(k + ":" + is_err)
            else:
                # 覆盖掉转换的值
                param_dict[k] = v

    # 日期字段格式转换
    if date_format_dict and isinstance(date_format_dict, dict):
        for k, v in param_dict.copy().items():
            if not date_format_dict.get(k):
                continue
            try:
                from_format, to_format = date_format_dict[k]
                if not isinstance(v, datetime.datetime) and isinstance(v, str):
                    parse_date_obj = datetime.datetime.strptime(v, from_format)
                else:
                    parse_date_obj = v
                param_dict[k] = parse_date_obj.strftime(to_format)
            except Exception as e:
                pass

    # 别名替换
    if alias_dict and isinstance(alias_dict, dict):
        param_dict = {alias_dict.get(k, k): v for k, v in param_dict.copy().items()}

    return param_dict


def filter_fields_handler(
        default_field_list: list = None,
        input_field_expression: 'str|list' = None,
        split_char: str = ";",
        all_field_list: list = None
) -> list:
    """
    过滤字段处理器
    使用：服务提供者只需要提供一个默认字段的列表或者符号分割的字符串，然后再把前端传进来的字段表达式传进来即可
    :param all_field_list: 全部合法性字段
    :param default_field_list: 默认字段列表,或者是符号分割的字符串
    :param input_field_expression: 字段处理表达式。"***filed_1;filed_2;;filed_2;!!!filed_1;filed_2;" 或者"filed_1;filed_2"或者 [filed_1,filed_2;]
    :param split_char: 拆分字符串,默认使用分号。
    :return: ["field_1",.....]
    """
    # all_field_list 与 default_field_list 因为是在服务层调用，所以强制列表类型
    if all_field_list is None or not isinstance(all_field_list, list):
        all_field_list = []
    if default_field_list is None or not isinstance(default_field_list, list):
        default_field_list = []
    # 处理默认字段
    default_field_list = default_field_list.split(split_char) if isinstance(default_field_list, str) else default_field_list or all_field_list

    # ========== 处理字段处理表达式 ==========
    # 如果没有传递字段表达式，默认字段不为空。则返回默认字段
    if input_field_expression is None:
        return default_field_list

    # 字段表达式字符串的情况
    elif isinstance(input_field_expression, str):
        if not re.search("[***|!!!]", input_field_expression):
            return format_list_handle(param_list=input_field_expression.split(split_char), filter_filed_list=all_field_list)
        # 加法或者减法原则
        default_field_hash = {i: "" for i in default_field_list}
        add_filed_expression = re.search("[***][^(!!!)]*", input_field_expression)
        sub_filed_expression = re.search("[!!!][^(***)]*", input_field_expression)
        if add_filed_expression:
            add_filed_list = add_filed_expression.group().replace("***", "").split(split_char)
            for i in add_filed_list:
                default_field_hash.update({i: ""})
        if sub_filed_expression:
            sub_filed_list = sub_filed_expression.group().replace("!!!", "").split(split_char)
            for i in sub_filed_list:
                default_field_hash.pop(i, None)
        return format_list_handle(param_list=[i for i in list(default_field_hash.keys()) if i], filter_filed_list=all_field_list)

    # 如果是列表则代表用户使用自定义的字段列表，不使用默认的字段列表
    elif isinstance(input_field_expression, list):
        return format_list_handle(param_list=input_field_expression, filter_filed_list=all_field_list)

    # input_field_expression的类型都不符合，则返回default_field_list
    else:
        return default_field_list
    # ========== 处理字段处理表达式 ==========


# 处理字典列表，白名单、黑名单、别名
def filter_result_field(
        result_list: list,
        filter_filed_list: list = None,
        remove_filed_list: list = None,
        alias_dict: dict = None,
        date_format_dict: dict = None
):
    """
    处理列表字典：白名单、黑名单、别名、移除空值、筛选类型等操作，把处理后的结果集返回出去。
    :param result_list: 结果集列表
    :param filter_filed_list: 白名单列表
    :param remove_filed_list: 黑名单列表
    :param alias_dict: 别名列表
    :param date_format_dict: 日期列表，{field：(from_format,to_format),}如：{"field":("%Y-%m-%d %H:%M:%S","%Y-%m-%d %H:%M:%S")}
    :return: result_list
    """
    # 转换的数据类型不符合，直接返回出去
    if not filter_filed_list and not remove_filed_list and not alias_dict:
        return result_list

    result = []
    for item in result_list:
        # 类型判断 过滤字段
        if filter_filed_list and isinstance(filter_filed_list, list):
            item = {k: v for k, v in item.copy().items() if k in filter_filed_list}
        # 类型判断， 剔除字段
        if remove_filed_list and isinstance(remove_filed_list, list):
            item = {k: v for k, v in item.copy().items() if not k in remove_filed_list}
        # 日期字段格式转换
        if date_format_dict and isinstance(date_format_dict, dict):
            for k, v in item.copy().items():
                if not date_format_dict.get(k):
                    continue
                try:
                    from_format, to_format = date_format_dict[k]
                    if not isinstance(v, datetime.datetime) and isinstance(v, str):
                        parse_date_obj = datetime.datetime.strptime(v, from_format)
                    else:
                        parse_date_obj = v
                    item[k] = parse_date_obj.strftime(to_format)
                except Exception as e:
                    pass
        # 类型判断 字段转换
        if alias_dict and isinstance(alias_dict, dict):
            item = {alias_dict.get(k, k): v for k, v in item.copy().items()}

        if item:
            result.append(item)

    return result


# 请求参数解析
def parse_data(request):
    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    content_type = request.META.get('CONTENT_TYPE', "").split(";")[0]
    method = request.method
    if content_type == "text/plain" or method == "GET":
        try:
            body = request.body.decode("utf-8")
            data = json.loads(body)
        except Exception:
            data = request.GET
            if not data:
                data = request.POST
            if not data:
                data = {}
    elif content_type == "application/json":
        return json.loads(request.body)
    elif content_type == "multipart/form-data":
        data = request.POST
    elif content_type == "application/xml":
        try:
            data = xmltodict.parse(request.body)
            return data.get("body") or data.get("data", {})
        except Exception as e:
            data = {}
    elif content_type == "application/x-www-form-urlencoded":
        data = parse_qs(request.body.decode())
        if data:
            data = {k: v[0] for k, v in data.items()}
        else:
            data = {}

    else:
        data = getattr(request, 'data', {})
    return {k: v for k, v in data.items()}


# 请求参数解析
def request_params_wrapper(func):
    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    def wrapper(instance, arg_request=None, *args, request=None, **kwargs):
        """
        解析request参数，适配多种body格式。
        PS :注意使用该装饰器之后必搭配*args，**kwargs须使用
        @param instance 实例是一个APIView的实例
        @param args 其它可变参数元组
        @param kwargs 其它可变关键字参数字典
        """
        if isinstance(instance, WSGIRequest) or isinstance(instance, Request) or isinstance(instance, ASGIRequest):
            request = instance
        if isinstance(arg_request, WSGIRequest) or isinstance(arg_request, Request) or isinstance(arg_request, ASGIRequest):
            request = arg_request
        if request is None:
            return func(instance, request, *args, request=request, request_params={}, **kwargs, )

        # 参数解析
        content_type = request.META.get('CONTENT_TYPE', "").split(";")[0]
        method = request.method
        if content_type == "text/plain" or method == "GET":  # 不指定则默认这种content-type
            try:
                body = request.body.decode("utf-8")
                data = json.loads(body)
            except Exception as e:
                # 允许get请求的query参数传json格式字符串，如：?group_list=["basics","bid-online"]
                data = parse_json(request.GET.dict())
                if not data:
                    data = request.POST
                if not data:
                    data = {}
        elif content_type == "application/json":
            data = json.loads(request.body)
        elif content_type == "multipart/form-data":
            data = request.POST
        elif content_type == "application/xml":
            try:
                data = xmltodict.parse(request.body)
                data = data.get("body") or data.get("data", {})
            except Exception as e:
                data = {}
        elif content_type == "application/x-www-form-urlencoded":
            data = parse_qs(request.body.decode())
            if data:
                data = {k: v[0] for k, v in data.items()}
            else:
                data = {}
        else:
            data = getattr(request, 'data', {})

        # 闭包抛出
        kwargs.pop("request_params", None)  # 避免重复执行报错
        kwargs.pop("request", None)  # 避免重复执行报错
        return func(instance, request, *args, request=request, request_params={k: v for k, v in data.items()}, **kwargs)

    return wrapper


# 随机发牌
def deal_equally(total: int, num: int):
    """
    发牌均分，发完截至, 然后打乱顺序
    如：5张牌发给三个人，则是2、2、1,然后打顺序，三个人中任何一个人可能得到一张牌。
    :param total: 总数
    :param num: 平均分配给这些人
    :return: list
    """
    every_one_jetton = int((total / num))
    overplus_jetton = total % num
    jetton_list = [every_one_jetton for i in range(num)]
    if overplus_jetton == 0:
        return jetton_list
    for index in range(overplus_jetton):
        jetton_list[index] = every_one_jetton + 1
    random.shuffle(jetton_list)
    return jetton_list


# 写入日志
def write_to_log(level="info", prefix="系统异常", content="", err_obj=None):
    """
    写入日志, 注意仅仅支持python3.0以上版本
    :param level: 写入错误日志等级
    :param prefix: 提示错误类型
    :param content: 错误内容
    :param err_obj: try except 捕捉到的错误对象
    :return: data, err_msg
    """
    logger = getLogger('log')
    try:
        if not err_obj is None:
            logger.error(
                '---' + prefix + ":" + str(err_obj) + ";" +
                (" content:" + str(content) + ";" if content else "") +
                " line:" + str(err_obj.__traceback__.tb_lineno) + ";" +
                " file:" + str(err_obj.__traceback__.tb_frame.f_globals["__file__"]) + ";" +
                " datetime:" + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time())) + ";"
            )
        elif level == "info":
            logger.error('---' + prefix + ":" + str(content))
        elif level == "error":
            logger.error('---' + prefix + ":" + str(content))
        return True, None
    except Exception as err:
        return False, str(err)


# 流程调用装饰器
def flow_service_wrapper(func):
    """
    API 流程中间件装饰器
    PS 该装饰器必须配套request_params_wrapper 和 user_wrapper 一起使用
    PS 开放的服务必须使用key_value类型接收参数，如：params: dict = None, **kwargs
    """

    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    def wrapper(instance, arg_request=None, *args, request=None, request_params=None, user_info: dict = None, **kwargs):
        """
        @param instance 实例是一个APIView的实例
        @param args 其它可变参数元组
        @param kwargs 其它可变关键字参数字典
        :param user_info: 用户信息，使用用户登录验证装饰器才会生效。
        :param request_params: 请求参数解析
        """
        if isinstance(instance, WSGIRequest) or isinstance(instance, Request) or isinstance(instance, ASGIRequest):
            request = instance
        if isinstance(arg_request, WSGIRequest) or isinstance(arg_request, Request) or isinstance(arg_request, ASGIRequest):
            request = arg_request

        # ----------------- section 得到请求参数以及用户信息 start --------------------------
        user_info, is_pass = force_transform_type(variable=user_info, var_type="dict", default={})
        request_params, is_pass = force_transform_type(variable=request_params, var_type="dict", default={})
        # 自动补全，用户信息字段
        for k, v in user_info.items():
            request_params.setdefault(k, v)
        # ----------------- section 得到请求参数以及用户信息 end   --------------------------

        # ----------------- section 加载流程类,并判断是否要执行流程 start --------------------------
        CopyFlowProcessService, is_import = dynamic_load_class(import_path="xj_flow.services.flow_process_service", class_name="FlowProcessService")
        # 检查请求参数中是否由流程相关信息,判断是触发流程
        flow_node_id = request_params.pop("flow_node_id", None)
        flow_action_id = request_params.pop("flow_action_id", None)
        flow_node_value = request_params.pop("flow_node_value", None)
        flow_action_value = request_params.pop("flow_action_value", None)

        if is_import or (not flow_node_id and not flow_node_value) or (not flow_action_id and not flow_action_value):
            return func(instance, *args, request=request, request_params=request_params, user_info=user_info.copy(), **kwargs)
        # ----------------- section 加载流程类,并判断是否要执行流程 end   --------------------------

        service = CopyFlowProcessService()
        # ----------------- section 流程阻断判断 start --------------------------
        data, is_prevent = service.flow_switch(flow_node_id=flow_node_id, flow_node_value=flow_node_value)
        if is_prevent:
            from .custom_response import util_response
            return util_response(err=5000, msg=is_prevent)
        # ----------------- section 流程阻断判断 end   --------------------------

        # ----------------- section 执行前置流程方法 start --------------------------
        data, err = service.do_once_flow_in_service(
            flow_node_id=flow_node_id,
            flow_node_value=flow_node_value,
            flow_action_id=flow_action_id,
            flow_action_value=flow_action_value,
            source_params=request_params.copy(),
            run_mode="BEFORE"
        )
        # 如果有错误则计入执行错误日志
        if err:
            write_to_log(prefix="流程装饰器调用异常:", content=err)
        # 处理后的参数默认值补全
        data = data or {}
        request_params = data.get("source_params", request_params)
        # ----------------- section 执行前置流程方法 end   --------------------------

        # ----------------- section 执行接口方法 start --------------------------
        response = func(instance, *args, request=request, request_params=request_params.copy(), user_info=user_info.copy(), **kwargs)
        # 获取http响应的内容
        try:
            response_json = parse_json(response.content.decode(encoding="utf-8"))
            response_json, is_pass = force_transform_type(variable=response_json, var_type="dict", default={})
            response_err = response_json.get("err", None)
            response_data, is_pass = force_transform_type(variable=response_json.get("data", {}), var_type="only_dict", default={})
        except ValueError:
            response_json = {}
            response_err = None
            response_data = {}
        # ----------------- section 执行接口方法 end   --------------------------

        # ----------------- section 执行后置流程方法 start --------------------------
        if not response_err:
            # 如果请求接口没有报错则不可以执行
            request_params.update(response_data)
            data, err = service.do_once_flow_in_service(
                flow_node_id=flow_node_id,
                flow_node_value=flow_node_value,
                flow_action_id=flow_action_id,
                flow_action_value=flow_action_value,
                source_params=request_params.copy(),
                run_mode="AFTER"
            )
            if err:
                write_to_log(prefix="流程装饰器调用异常:", content=err)
        # ----------------- section 执行后置流程方法 end   --------------------------

        # ----------------- section 记录流程记录 start --------------------------
        # 流程完成记录
        if not response_err:
            data, flow_err = service.finish_flow(user_id=user_info.get("user_id"))
            if flow_err:
                write_to_log(prefix="流程装饰器保存流程记录失败", content=flow_err)
        # 规则执行记录
        record, record_err = service.save_record(result_dict=response_json.get("data", {}), user_info=user_info.copy())
        if record_err:
            write_to_log(prefix="流程装饰器保存流程记录失败", content=record_err)
        # ----------------- section 记录流程记录 end   --------------------------
        return response

    return wrapper


# 计算经纬度距离
def geodistance(lng1, lat1, lng2, lat2):
    """
    计算经纬度距离
    :param lng1: 经度1
    :param lat1: 维度1
    :param lng2: 经度2
    :param lat2: 维度2
    :return: 计算距离
    """
    lng1, lat1, lng2, lat2 = map(radians, [float(lng1), float(lat1), float(lng2), float(lat2)])  # 经纬度转换成弧度
    dlon = lng2 - lng1
    dlat = lat2 - lat1
    a = sin(dlat / 2) ** 2 + cos(lat1) * cos(lat2) * sin(dlon / 2) ** 2
    distance = 2 * asin(sqrt(a)) * 6371 * 1000  # 地球平均半径，6371km
    distance = round(distance / 1000, 3) * 1000
    return distance


# 计算文本相似度
def string_similar(s1, s2):
    """
    计算字符串s1与s2的差别
    :param s1: 字符串1
    :param s2: 字符串2
    :return: 两者差异度相同位1
    """
    return difflib.SequenceMatcher(None, s1, s2).ratio()


# 生成随机字符串，2千万次重复两次。
def get_short_id(length=8):
    """
    生成随机字符串，2千万次重复两次。
    :param length:
    :return: 随机字符串
    """
    length = 8 if length > 8 else length
    dictionary = [
        "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
        "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z",
        "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"
    ]
    random_id = str(uuid.uuid4()).replace("-", '')  # 注意这里需要用uuid4
    buffer = []
    for i in range(0, length):
        start = i * 4
        end = i * 4 + 4
        val = int(random_id[start:end], 16)
        buffer.append(dictionary[val % 62])
    return "".join(buffer)


# 字典合并字段冲突，自动加前缀
def conflict_fields(source_data: "list|dict" = None, foreign_data: "list|dict" = None, prefix=""):
    """
    字典合并字段冲突，自动加前缀
    @note 注意仅仅支持{}或者[{}]格式
    :param prefix: 字段冲突添加前缀
    :param source_data: 源数据集合
    :param foreign_data: 外来数据集合
    :return: foreign_data
    """
    if (not isinstance(foreign_data, list) and not isinstance(foreign_data, dict)) or (not isinstance(source_data, list) and not isinstance(source_data, dict)):
        return foreign_data

    # 获取源数据字段名称
    source_keys = []
    if isinstance(source_data, list):
        for item in source_data:
            if isinstance(item, dict):
                source_keys += list(item.keys())
        source_keys = list(set(source_keys))
    else:
        source_keys = list(source_data.keys())

    # 获取外来数据字段key
    foreign_keys = []
    if isinstance(foreign_data, list):
        for item in foreign_data:
            if isinstance(item, dict):
                foreign_keys += list(item.keys())
        foreign_keys = list(set(foreign_keys))
    else:
        foreign_keys = list(foreign_data.keys())

    # 建立外键的映射
    conflict_fields_map = {i: str(prefix) + i for i in foreign_keys if i in source_keys}

    # 冲突字段添加前缀替换
    if isinstance(foreign_data, list):
        foreign_data = filter_result_field(
            result_list=foreign_data,
            alias_dict=conflict_fields_map
        )
    else:
        foreign_data = format_params_handle(
            param_dict=foreign_data,
            alias_dict=conflict_fields_map
        )
    return foreign_data
